# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import time
import pymongo
import superdesk

from flask import current_app as app
from superdesk.errors import BulkIndexError
from superdesk import config


class IndexFromMongo(superdesk.Command):
    """Index the specified mongo collection in the specified elastic collection/type.

    This will use the default APP mongo DB to read the data and the default Elastic APP index.

    Use ``-f all`` to index all collections.

    Example:
    ::

        $ python manage.py app:index_from_mongo --from=archive
        $ python manage.py app:index_from_mongo --all

    """

    option_list = [
        superdesk.Option("--from", "-f", dest="collection_name"),
        superdesk.Option("--all", action="store_true", dest="all_collections"),
        superdesk.Option("--page-size", "-p"),
    ]
    default_page_size = 500

    def run(self, collection_name, all_collections, page_size):
        if not collection_name and not all_collections:
            raise SystemExit("Specify --all to index from all collections")
        elif all_collections:
            app.data.init_elastic(app)
            resources = app.data.get_elastic_resources()
            for resource in resources:
                self.copy_resource(resource, page_size)
        else:
            self.copy_resource(collection_name, page_size)

    @classmethod
    def copy_resource(cls, resource, page_size):
        for items in cls.get_mongo_items(resource, page_size):
            print("{} Inserting {} items".format(time.strftime("%X %x %Z"), len(items)))
            s = time.time()
            success, failed = 0, 0

            for i in range(1, 4):
                try:
                    success, failed = superdesk.app.data._search_backend(resource).bulk_insert(resource, items)
                except Exception as ex:
                    print("Exception thrown on insert to elastic {}", ex)
                    time.sleep(10)
                    continue
                else:
                    break

            print("{} Inserted {} items in {:.3f} seconds".format(time.strftime("%X %x %Z"), success, time.time() - s))
            if failed:
                print("Failed to do bulk insert of items {}. Errors: {}".format(len(failed), failed))
                raise BulkIndexError(resource=resource, errors=failed)

        return "Finished indexing collection {}".format(resource)

    @classmethod
    def get_mongo_items(cls, mongo_collection_name, page_size):
        """Generate list of items from given mongo collection per page size.

        :param mongo_collection_name: Name of the collection to get the items
        :param page_size: Size of every list in each iteration
        :return: list of items
        """
        bucket_size = int(page_size) if page_size else cls.default_page_size
        print("Indexing data from mongo/{} to elastic/{}".format(mongo_collection_name, mongo_collection_name))

        db = app.data.get_mongo_collection(mongo_collection_name)
        args = {"limit": bucket_size, "sort": [(config.ID_FIELD, pymongo.ASCENDING)]}
        last_id = None
        while True:
            if last_id:
                args.update({"filter": {config.ID_FIELD: {"$gt": last_id}}})
            cursor = db.find(**args)
            if not cursor.count():
                break
            items = list(cursor)
            last_id = items[-1][config.ID_FIELD]
            yield items


superdesk.command("app:index_from_mongo", IndexFromMongo())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
from flask import current_app as app
from superdesk import get_resource_service
from superdesk.services import BaseService
from eve.utils import ParsedRequest
from superdesk.notification import push_notification
from apps.archive.common import get_user
from eve.utils import config
from superdesk.utc import utcnow
from apps.archive.common import ITEM_MARK, ITEM_UNMARK


def get_marked_items(desk_id):
    """Get items marked for given desk"""
    query = {
        "query": {"filtered": {"filter": {"term": {"marked_desks.desk_id": str(desk_id)}}}},
        "sort": [{"versioncreated": "desc"}],
        "size": 200,
    }
    request = ParsedRequest()
    request.args = {"source": json.dumps(query), "repo": "archive,published"}
    return list(get_resource_service("search").get(req=request, lookup=None))


class MarkedForDesksService(BaseService):
    def create(self, docs, **kwargs):
        """Toggle marked desk status for given desk and item."""

        service = get_resource_service("archive")
        published_service = get_resource_service("published")
        ids = []
        for doc in docs:
            item = service.find_one(req=None, guid=doc["marked_item"])
            if not item:
                ids.append(None)
                continue
            ids.append(item["_id"])
            marked_desks = item.get("marked_desks", [])
            if not marked_desks:
                marked_desks = []

            existing_mark = next((m for m in marked_desks if m["desk_id"] == doc["marked_desk"]), None)

            if existing_mark:
                # there is an existing mark so this is un-mark action
                marked_desks = [m for m in marked_desks if m["desk_id"] != doc["marked_desk"]]
                marked_desks_on = False  # highlight toggled off
            else:
                # there is no existing mark so this is mark action
                user = get_user() or {}
                new_mark = {}
                new_mark["desk_id"] = doc["marked_desk"]
                new_mark["user_marked"] = str(user.get(config.ID_FIELD, ""))
                new_mark["date_marked"] = utcnow()
                marked_desks.append(new_mark)
                marked_desks_on = True

            updates = {"marked_desks": marked_desks}
            service.system_update(item["_id"], updates, item)

            publishedItems = published_service.find({"item_id": item["_id"]})
            for publishedItem in publishedItems:
                if publishedItem["_current_version"] == item["_current_version"] or not marked_desks_on:
                    updates = {"marked_desks": marked_desks}
                    published_service.system_update(publishedItem["_id"], updates, publishedItem)

            push_notification(
                "item:marked_desks", marked=int(marked_desks_on), item_id=item["_id"], mark_id=str(doc["marked_desk"])
            )

            if marked_desks_on:
                app.on_archive_item_updated({"desk_id": doc["marked_desk"]}, item, ITEM_MARK)
            else:
                app.on_archive_item_updated({"desk_id": doc["marked_desk"]}, item, ITEM_UNMARK)

        return ids



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from functools import partial
from unittest.mock import MagicMock, patch
from .utils import mock_dictionaries
import responses
from flask import Flask
from superdesk.tests import TestCase
from superdesk.text_checkers import tools
from superdesk.text_checkers.spellcheckers.base import registered_spellcheckers, SpellcheckerBase
from superdesk.text_checkers import spellcheckers
from superdesk.text_checkers.spellcheckers.leuven_dutch import LeuvenDutch, API_URL, OPT_API_KEY
from superdesk import get_resource_service

spellcheckers.AUTO_IMPORT = False
MODEL = {
    "outo": 1,
}


@responses.activate
def load_spellcheckers():
    registered_spellcheckers.clear()
    app = Flask(__name__)
    app.config[OPT_API_KEY] = "123-456-789-ABC"
    tools.import_services(app, spellcheckers.__name__, SpellcheckerBase)


class LeuvenDutchTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_spellcheckers()

    def test_list(self):
        """Check that Leuven Dutch spellchecker is listed by spellcheckers_list service"""
        doc = {}
        spellcheckers_list = get_resource_service("spellcheckers_list")
        spellcheckers_list.on_fetched(doc)
        for checker in doc["spellcheckers"]:
            if (
                checker["name"] == LeuvenDutch.name
                and "label" in checker
                and set(checker["capacities"]) == set(LeuvenDutch.capacities)
                and checker["languages"] == ["nl"]
            ):
                return
        self.fail("Leuven University Dutch spellchecker not found")

    @responses.activate
    def test_checker(self, expected=None, use_internal_dict=False):
        """Check that spellchecking is working

        :param expected: "errors" expected
            to be specified if this test is re-used
        :param use_internal_dict: value to use in request
            can be changed if this test is re-used
        """
        doc = {
            "spellchecker": LeuvenDutch.name,
            "text": "Outo rijden is gevaarlijk",
            "suggestions": False,
            "use_internal_dict": use_internal_dict,
        }
        spellchecker = get_resource_service("spellchecker")
        check_url = API_URL.format(method="spellingchecker")
        responses.add(
            responses.POST,
            check_url,
            json={
                "status": {"message": "ok", "code": 200},
                "settings": {"startmarker": "##__ERR_START__##", "endmarker": "##__ERR_END__##", "html": False},
                "spellingchecker": {
                    "input": "Outo rijden is gevaarlijk",
                    "output": {
                        "context": [""],
                        "mistakes": ["Outo"],
                        "marked": "##__ERR_START__##Outo##__ERR_END__## rijden is gevaarlijk",
                    },
                },
                "debug": {"timeneeded": 0.1197},
            },
        )
        spellchecker.create([doc])

        if expected is None:
            expected = [{"startOffset": 0, "text": "Outo", "type": "spelling"}]

        self.assertEqual(doc["errors"], expected)

    @patch("superdesk.get_resource_service", MagicMock(side_effect=partial(mock_dictionaries, model=MODEL)))
    def test_checker_with_internal_dict(self):
        """Check that words in personal dictionary are discarded correctly

        This test re-uses test_checker but activate "use_internal_dict" option and mock dictionary to have "outo" inside
        """
        # outo is in personal dictionary, so no spelling error should be returned
        expected = []
        self.test_checker(expected=expected, use_internal_dict=True)

    @responses.activate
    def test_suggest(self):
        """Check that spelling suggestions are working"""

        doc = {"spellchecker": "leuven_dutch", "text": "Outo", "suggestions": True}
        spellchecker = get_resource_service("spellchecker")
        check_url = API_URL.format(method="suggesties")
        responses.add(
            responses.POST,
            check_url,
            json={
                "status": {"message": "ok", "code": 200},
                "suggesties": {
                    "input": "Outo",
                    "output": {
                        "suggesties": ["Otto", "Buto", "Kato", "Cato", "Lato", "Leto", "Curto", "Mato", "Beto", "Busto"]
                    },
                },
                "debug": {"timeneeded": 0.0861},
            },
        )
        spellchecker.create([doc])

        self.assertEqual(
            doc,
            {
                "spellchecker": "leuven_dutch",
                "suggestions": [
                    {"text": "Otto"},
                    {"text": "Buto"},
                    {"text": "Kato"},
                    {"text": "Cato"},
                    {"text": "Lato"},
                    {"text": "Leto"},
                    {"text": "Curto"},
                    {"text": "Mato"},
                    {"text": "Beto"},
                    {"text": "Busto"},
                ],
                "text": "Outo",
            },
        )



from unittest.mock import patch
from bson import ObjectId
from datetime import timedelta

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from apps.auth.session_purge import RemoveExpiredSessions
from apps.auth import is_current_user_admin


class AuthTestCase(TestCase):
    def test_remove_expired_sessions_syncs_online_users(self):
        sess_id = ObjectId()
        user_ids = self.app.data.insert(
            "users",
            [
                {"name": "foo", "username": "foo", "session_preferences": {"test": {}}},
                {"name": "bar", "username": "bar", "session_preferences": {"bar": {}, str(sess_id): {}}},
            ],
        )
        self.assertEqual(2, len(user_ids))

        self.app.data.insert(
            "auth",
            [
                {"user": user_ids[0], "_updated": utcnow() - timedelta(days=50)},
                {"user": user_ids[1], "_updated": utcnow()},
                {"_id": sess_id, "user": user_ids[1], "_updated": utcnow()},
            ],
        )
        _ids = [_auth["user"] for _auth in self.app.data.find_all("auth")]
        self.assertEqual(3, len(_ids))

        RemoveExpiredSessions().run()

        # don't expose user preferences
        users = self.app.data.find_list_of_ids("users", user_ids)
        self.assertNotIn("session_preferences", users[0])
        self.assertNotIn("session_preferences", users[1])
        # ensure that expired auth tokens were removed
        _ids = [_auth["user"] for _auth in self.app.data.find_all("auth")]
        self.assertEqual(2, len(_ids))
        self.assertEqual(user_ids[1], _ids[0])
        self.assertEqual(user_ids[1], _ids[1])

    def test_is_current_user_admin(self):
        with patch("apps.auth.get_user", return_value={}):
            self.assertFalse(is_current_user_admin())

        with patch("apps.auth.get_user", return_value={"user_type": "administrator"}):
            self.assertTrue(is_current_user_admin())

    def test_session_expiry_date_update(self):
        user_ids = self.app.data.insert(
            "users",
            [
                {"username": "foo", "user_type": "administrator"},
            ],
        )
        initial_updated = utcnow() - timedelta(minutes=5)
        self.app.data.insert(
            "auth",
            [
                {"user": user_ids[0], "_updated": initial_updated, "token": "foo"},
            ],
        )

        with self.app.test_request_context("/users", method="POST"):
            self.app.auth.check_auth("foo", [], "users", "POST")
            auth = self.app.data.find_one("auth", None, token="foo")
            self.assertGreaterEqual(auth["_updated"], utcnow() - timedelta(seconds=1))
            user = self.app.data.find_one("users", req=None, username="foo")
            self.assertEqual(user["last_activity_at"], auth["_updated"])

            self.app.data.update("auth", auth["_id"], {"_updated": utcnow() - timedelta(seconds=10)}, auth)

            self.app.auth.check_auth("foo", [], "users", "POST")
            auth = self.app.data.find_one("auth", None, token="foo")
            self.assertLess(auth["_updated"], utcnow() - timedelta(seconds=1))



import flask
import unittest
import superdesk.tests as tests
import superdesk.auth.saml as saml

from unittest.mock import patch


SAML_DATA = {
    "http://schemas.microsoft.com/identity/claims/tenantid": ["3cd37c1d"],
    "http://schemas.microsoft.com/identity/claims/objectidentifier": ["2f2a99f7"],
    "http://schemas.microsoft.com/identity/claims/displayname": ["Bar, Foo"],
    "http://schemas.microsoft.com/identity/claims/identityprovider": ["https://sts.windows.net/3cd37c1d/"],
    "http://schemas.microsoft.com/claims/authnmethodsreferences": [
        "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport",
    ],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname": ["Foo"],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname": ["Bar"],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress": ["foo.bar@example.com"],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": ["foo@bar.com"],
    "http://schemas.microsoft.com/ws/2008/06/identity/claims/role": ["editor"],
    "displayname": ["Bar, Foo"],
    "country": ["CA"],
    "city": ["Toronto"],
    "department": ["IT"],
}


ERROR = '{"error": 404}'


class SamlAuthTestCase(tests.TestCase):
    def setUp(self):
        self.desks = [{"name": "Sports"}, {"name": "Finance"}]
        self.roles = [{"name": "Editor"}, {"name": "Admin"}]
        with self.app.app_context():
            self.app.data.insert("desks", self.desks)
            self.app.data.insert("roles", self.roles)

    @patch("superdesk.auth.saml.init_saml_auth")
    def test_create_missing_user(self, init_mock):
        with self.app.test_client() as c:
            flask.session[saml.SESSION_NAME_ID] = "foo.bar@example.com"
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA

            resp = saml.index()
            self.assertIn(ERROR, resp)

            with patch.dict(
                self.app.config,
                {
                    "USER_EXTERNAL_CREATE": True,
                    "USER_EXTERNAL_DESK": "sports",
                    "USER_EXTERNAL_USERNAME_STRIP_DOMAIN": True,
                },
            ):
                resp = saml.index()
            self.assertNotIn(ERROR, resp)

            user = self.app.data.find_one("users", req=None, email="foo.bar@example.com")
            self.assertTrue(user.get("is_active"))
            self.assertTrue(user.get("is_enabled"))
            self.assertFalse(user.get("needs_activation"))
            self.assertEqual("user", user.get("user_type"))
            self.assertEqual("foo", user["username"])

            self.assertIsNotNone(user.get("role"))
            role = self.app.data.find_one("roles", req=None, _id=user["role"])
            self.assertEqual("Editor", role["name"])

            self.assertIsNotNone(user.get("desk"))
            desk = self.app.data.find_one("desks", req=None, _id=user["desk"])
            self.assertIn({"user": user["_id"]}, desk.get("members"))

    @patch("superdesk.auth.saml.init_saml_auth")
    def test_create_missing_user_missing_userdata(self, init_mock):
        with self.app.test_client() as c:
            # with missing data it can't work
            flask.session[saml.SESSION_NAME_ID] = "foo.bar@example.com"
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA.copy()
            flask.session[saml.SESSION_USERDATA_KEY].update(
                {
                    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": [],
                }
            )
            with patch.dict(self.app.config, {"USER_EXTERNAL_CREATE": True}):
                resp = saml.index()
            self.assertIn(ERROR, resp)

    @patch("superdesk.auth.saml.init_saml_auth")
    def test_handle_saml_name_id_not_email(self, init_mock):
        with self.app.test_client() as c:
            # with missing data it can't work
            flask.session[saml.SESSION_NAME_ID] = "something_weird_like_guid"
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA.copy()
            with patch.dict(self.app.config, {"USER_EXTERNAL_CREATE": True}):
                resp = saml.index()
            self.assertNotIn(ERROR, resp)

    @patch("superdesk.auth.saml.init_saml_auth")
    def test_update_user_data_when_it_changes(self, init_mock):
        with self.app.test_client() as c:
            # with missing data it can't work
            flask.session[saml.SESSION_NAME_ID] = "nameId"
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA.copy()
            with patch.dict(
                self.app.config,
                {
                    "USER_EXTERNAL_CREATE": True,
                    "USER_EXTERNAL_DESK": "sports",
                },
            ):
                resp = saml.index()

            user = self.app.data.find_one("users", req=None, email="foo.bar@example.com")
            self.assertIsNotNone(user)
            self.assertEqual(self.desks[0]["_id"], user["desk"])

            self.app.data.update(
                "users",
                user["_id"],
                {
                    "user_type": "administrator",
                    "desk": self.desks[1]["_id"],
                },
                user,
            )

            flask.session[saml.SESSION_USERDATA_KEY].update(
                {
                    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname": ["John"],
                    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname": ["Doe"],
                    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress": ["foo.bar@example.com"],
                    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": ["john"],
                    "displayname": ["Doe, John"],
                }
            )

            with patch.dict(
                self.app.config,
                {
                    "USER_EXTERNAL_CREATE": True,
                    "USER_EXTERNAL_DESK": "sports",
                },
            ):
                resp = saml.index()

        user = self.app.data.find_one("users", req=None, email="foo.bar@example.com")
        self.assertEqual("John", user["first_name"])
        self.assertEqual("Doe", user["last_name"])
        self.assertEqual("Doe, John", user["display_name"])
        self.assertEqual("administrator", user["user_type"])
        self.assertEqual(self.desks[1]["_id"], user["desk"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from enum import Enum
from lxml import etree
from superdesk.text_utils import get_text
from superdesk.utc import utcnow
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError
from flask_babel import _


class FilterConditionFieldsEnum(Enum):
    anpa_category = (1,)
    genre = (2,)
    subject = (3,)
    desk = (4,)
    sms = (5,)
    urgency = (6,)
    priority = (7,)
    type = (8,)
    keywords = (9,)
    slugline = (10,)
    source = (11,)
    headline = (12,)
    body_html = (13,)
    stage = (14,)
    ednote = (15,)
    place = (16,)
    ingest_provider = 17
    embargo = 18
    featuremedia = 19
    agendas = 20


class FilterConditionField:
    @staticmethod
    def factory(field):
        if field not in FilterConditionFieldsEnum.__members__:
            vocabulary = get_resource_service("vocabularies").find_one(req=None, _id=field)
            if vocabulary:
                if vocabulary.get("field_type", "") == "text":
                    return FilterConditionCustomTextField(field)
                else:
                    return FilterConditionControlledVocabularyField(field)
            raise SuperdeskApiError.internalError(_("Invalid filter conditions field {field}").format(field=field))
        if FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.desk:
            return FilterConditionDeskField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.stage:
            return FilterConditionStageField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.anpa_category:
            return FilterConditionCategoryField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.genre:
            return FilterConditionGenreField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.sms:
            return FilterConditionSmsField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.subject:
            return FilterConditionSubjectField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.urgency:
            return FilterConditionUrgencyField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.priority:
            return FilterConditionPriorityField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.place:
            return FilterConditionPlaceField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.ingest_provider:
            return FilterConditionIngestProviderField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.embargo:
            return FilterConditionEmbargoField(field)
        elif FilterConditionFieldsEnum[field] == FilterConditionFieldsEnum.featuremedia:
            return FilterConditionFeatureMediaField(field)
        else:
            return FilterConditionField(field)

    def __init__(self, field):
        self.field = FilterConditionFieldsEnum[field]
        self.entity_name = field
        self.field_type = str

    def get_entity_name(self):
        return self.entity_name

    def get_type(self):
        return self.field_type

    def is_in_article(self, article):
        return self.field.name in article and article.get(self.field.name) is not None

    def get_value(self, article):
        try:
            return get_text(article[self.field.name]).strip()
        except (etree.XMLSyntaxError, ValueError):
            return article[self.field.name]


class FilterConditionDeskField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.desk
        self.entity_name = "task.desk"
        self.field_type = str

    def is_in_article(self, article):
        return self.field.name in article.get("task", {})

    def get_value(self, article):
        return str(article.get("task", {}).get(self.field.name))


class FilterConditionStageField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.stage
        self.entity_name = "task.stage"
        self.field_type = str

    def is_in_article(self, article):
        return self.field.name in article.get("task", {})

    def get_value(self, article):
        return str(article.get("task", {}).get(self.field.name))


class FilterConditionSubjectField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.subject
        self.entity_name = "subject.qcode"
        self.field_type = str

    def get_value(self, article):
        return [s["qcode"] for s in article[self.field.name]]


class FilterConditionCategoryField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.anpa_category
        self.entity_name = "anpa_category.qcode"
        self.field_type = str

    def get_value(self, article):
        return [c["qcode"] for c in article[self.field.name]]


class FilterConditionGenreField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.genre
        self.entity_name = "genre.name"
        self.field_type = str

    def get_value(self, article):
        return [g["name"] for g in article[self.field.name]]


class FilterConditionUrgencyField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum[field]
        self.entity_name = field
        self.field_type = int


class FilterConditionPriorityField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum[field]
        self.entity_name = field
        self.field_type = int


class FilterConditionSmsField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.sms
        self.entity_name = "flags.marked_for_sms"
        self.field_type = bool

    def is_in_article(self, article):
        return "marked_for_sms" in article.get("flags", {})

    def get_value(self, article):
        return str(article.get("flags", {}).get("marked_for_sms"))


class FilterConditionPlaceField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.place
        self.entity_name = "place.qcode"
        self.field_type = str

    def get_value(self, article):
        return [c["qcode"] for c in article[self.field.name]]


class FilterConditionIngestProviderField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.ingest_provider
        self.entity_name = "ingest_provider"
        self.field_type = str


class FilterConditionEmbargoField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.embargo
        self.entity_name = "embargo"
        self.field_type = bool

    def is_in_article(self, article):
        return article.get("embargo")

    def get_value(self, article):
        if article.get("embargo"):
            utc_embargo = article.get("schedule_settings", {}).get("utc_embargo")
            if utc_embargo and utc_embargo > utcnow():
                return str(True)
        return str(False)

    def get_elastic_query(self):
        return {"range": {"schedule_settings.utc_embargo": {"gt": utcnow().isoformat()}}}

    def get_mongo_query(self):
        return {"schedule_settings.utc_embargo": {"$gt": utcnow()}}


class FilterConditionControlledVocabularyField(FilterConditionField):
    def __init__(self, field):
        self.field = type("_ControlledVocabulary", (object,), {"name": field})()
        self.entity_name = "subject.qcode"
        self.field_type = str

    def is_in_article(self, article):
        return any([self.field.name == subject.get("scheme") for subject in article.get("subject", [])])

    def get_value(self, article):
        return [s["qcode"] for s in article["subject"]]


class FilterConditionCustomTextField(FilterConditionField):
    def __init__(self, field):
        self.field = type("_CustomTextField", (object,), {"name": field})()
        self.entity_name = "extra"
        self.field_type = str

    def is_in_article(self, article):
        return self.field.name in article.get("extra", {})

    def get_value(self, article):
        return article["extra"].get(self.field.name, "")


class FilterConditionFeatureMediaField(FilterConditionField):
    def __init__(self, field):
        self.field = FilterConditionFieldsEnum.featuremedia
        self.entity_name = "associations.featuremedia._id"
        self.field_type = bool

    def get_value(self, article):
        return ((article.get("associations") or {}).get("featuremedia") or {}).get("_id")

    def is_in_article(self, article):
        return "_id" in ((article.get("associations") or {}).get("featuremedia") or {})

    def get_elastic_query(self):
        return {"exists": {"field": "associations.featuremedia._id"}}

    def get_mongo_query(self):
        return {"associations.featuremedia._id": {"$exists": True}}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk import get_backend

from apps.content_filters.filter_condition.filter_condition_resource import FilterConditionResource
from apps.content_filters.filter_condition.filter_condition_service import FilterConditionService
from apps.content_filters.content_filter.content_filter_resource import ContentFilterResource
from apps.content_filters.content_filter.content_filter_service import ContentFilterService

from apps.content_filters.filter_condition.filter_condition_parameters import (
    FilterConditionParametersResource,
    FilterConditionParametersService,
)

from apps.content_filters.content_filter.content_filter_test import ContentFilterTestResource, ContentFilterTestService


def init_app(app):
    endpoint_name = "filter_conditions"
    service = FilterConditionService(endpoint_name, backend=get_backend())
    FilterConditionResource(endpoint_name, app=app, service=service)

    endpoint_name = "filter_condition_parameters"
    service = FilterConditionParametersService(endpoint_name, backend=get_backend())
    FilterConditionParametersResource(endpoint_name, app=app, service=service)

    endpoint_name = "content_filters"
    service = ContentFilterService(endpoint_name, backend=get_backend())
    ContentFilterResource(endpoint_name, app=app, service=service)

    endpoint_name = "content_filter_tests"
    service = ContentFilterTestService(endpoint_name, backend=superdesk.get_backend())
    ContentFilterTestResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name="content_filters", label="Content Filters", description="User can manage content filters")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.macros.macro_register import register, load_macros  # NOQA
from apps.macros.macros import MacrosResource, MacrosService


def init_app(app):
    MacrosResource("macros", app=app, service=MacrosService())
    load_macros(app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


"""Media archive module"""

import logging

import superdesk
from superdesk.celery_app import celery

from .archive import (
    ArchiveResource,
    ArchiveService,
    ArchiveVersionsResource,
    ArchiveVersionsService,
    AutoSaveResource,
    ArchiveSaveService,
)
from .commands import RemoveExpiredContent
from .ingest import IngestResource, AppIngestService
from .user_content import UserContentResource, UserContentService
from .archive_lock import ArchiveLockResource, ArchiveUnlockResource, ArchiveLockService, ArchiveUnlockService
from .archive_spike import ArchiveUnspikeResource, ArchiveSpikeService, ArchiveSpikeResource, ArchiveUnspikeService
from .archive_correction import ArchiveCorrectionService, ArchiveCorrectionResource
from .related import ArchiveRelatedResource, ArchiveRelatedService
from apps.common.components.utils import register_component
from apps.item_lock.components.item_lock import ItemLock
from apps.common.models.utils import register_model
from apps.item_lock.models.item import ItemModel
from apps.common.models.io.eve_proxy import EveProxy
from .archive_rewrite import ArchiveRewriteResource, ArchiveRewriteService
from .news import NewsResource, NewsService
from flask_babel import _

logger = logging.getLogger(__name__)


def init_app(app):

    endpoint_name = "ingest"
    service = AppIngestService(endpoint_name, backend=superdesk.get_backend())
    IngestResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_versions"
    service = ArchiveVersionsService(endpoint_name, backend=superdesk.get_backend())
    ArchiveVersionsResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive"
    service = ArchiveService(endpoint_name, backend=superdesk.get_backend())
    ArchiveResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_lock"
    service = ArchiveLockService(endpoint_name, backend=superdesk.get_backend())
    ArchiveLockResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_unlock"
    service = ArchiveUnlockService(endpoint_name, backend=superdesk.get_backend())
    ArchiveUnlockResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_spike"
    service = ArchiveSpikeService(endpoint_name, backend=superdesk.get_backend())
    ArchiveSpikeResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_unspike"
    service = ArchiveUnspikeService(endpoint_name, backend=superdesk.get_backend())
    ArchiveUnspikeResource(endpoint_name, app=app, service=service)

    endpoint_name = "user_content"
    service = UserContentService(endpoint_name, backend=superdesk.get_backend())
    UserContentResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_rewrite"
    service = ArchiveRewriteService(endpoint_name, backend=superdesk.get_backend())
    ArchiveRewriteResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_correction"
    service = ArchiveCorrectionService(endpoint_name, backend=superdesk.get_backend())
    ArchiveCorrectionResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_autosave"
    service = ArchiveSaveService(endpoint_name, backend=superdesk.get_backend())
    AutoSaveResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_related"
    service = ArchiveRelatedService(endpoint_name, backend=superdesk.get_backend())
    ArchiveRelatedResource(endpoint_name, app=app, service=service)

    endpoint_name = "news"
    service = NewsService(endpoint_name, backend=superdesk.get_backend())
    NewsResource(endpoint_name, app=app, service=service)

    from apps.item_autosave.components.item_autosave import ItemAutosave
    from apps.item_autosave.models.item_autosave import ItemAutosaveModel

    register_component(ItemLock(app))
    register_model(ItemModel(EveProxy(superdesk.get_backend())))
    register_component(ItemAutosave(app))
    register_model(ItemAutosaveModel(EveProxy(superdesk.get_backend())))

    superdesk.privilege(
        name="monitoring_view", label=_("Monitoring view"), description=_("Access to Monitoring view in left toolbar")
    )
    superdesk.privilege(name="archive", label=_("Create content"), description=_("Create and save content"))
    superdesk.privilege(name="ingest", label=_("Ingest"), description=_("Access to ingest sources management"))
    superdesk.privilege(name="spike", label=_("Spike"), description=_("Spike/delete items"))
    superdesk.privilege(name="spike_read", label=_("Spike view"), description=_("View spiked content"))
    superdesk.privilege(name="unspike", label=_("Unspike"), description=_("Unspike/undelete content"))
    superdesk.privilege(name="metadata_uniquename", label=_("Edit Unique Name"), description=_("Edit unique name"))
    superdesk.privilege(name="hold", label=_("Hold"), description=_("Hold content"))
    superdesk.privilege(name="restore", label=_("Restore"), description=_("Restore content"))
    superdesk.privilege(name="rewrite", label=_("Update"), description=_("Create an update"))
    superdesk.privilege(
        name="unlock", label=_("Unlock content"), description=_("Unlock locked content by another user")
    )
    superdesk.privilege(
        name="mark_for_user",
        label=_("Mark items for users"),
        description=_("User can mark or unmark items for other users"),
    )

    superdesk.intrinsic_privilege(ArchiveUnlockResource.endpoint_name, method=["POST"])


@celery.task(soft_time_limit=3600)
def content_expiry():
    RemoveExpiredContent().run()



from superdesk.resource import Resource

TODAY_DATE = "now/d"
WEEK_DATE = "now/w"

allowed_times = ["now-{0}h".format(hour) for hour in range(1, 25)]
allowed_times.append(TODAY_DATE)
allowed_times.append(WEEK_DATE)


class HighlightsResource(Resource):
    """Highlights schema"""

    schema = {
        "name": {"type": "string", "iunique": True, "required": True},
        "template": Resource.rel("content_templates", nullable=True),
        "desks": {"type": "list", "schema": Resource.rel("desks", True)},
        "auto_insert": {
            "type": "string",
            "allowed": allowed_times,
            "default": TODAY_DATE,
        },
        "groups": {"type": "list", "schema": {"type": "string"}},
    }
    privileges = {"POST": "highlights", "PATCH": "highlights", "DELETE": "highlights"}

    mongo_indexes = {
        "name_1": ([("name", 1)], {"unique": True}),
    }


class MarkedForHighlightsResource(Resource):
    """Marked for highlights Schema"""

    schema = {"highlights": {"type": "string", "required": True}, "marked_item": {"type": "string", "required": True}}
    privileges = {"POST": "mark_for_highlights"}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase
from unittest import mock


class SuperdeskValidatorTest(TestCase):
    """Base class for the SuperdeskValidator class tests."""

    def _get_target_class(self):
        """Return the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from superdesk.validator import SuperdeskValidator
        except ImportError:
            self.fail("Could not import class under test (SuperdeskValidator)")
        else:
            return SuperdeskValidator


@mock.patch("superdesk.validator.superdesk")
class ValidateIuniqueMethodTestCase(SuperdeskValidatorTest):
    """Tests for the _validate_iunique() method."""

    def setUp(self):
        klass = self._get_target_class()
        self.validator = klass(schema={})

    def test_does_not_raise_error_on_inputs_containing_special_regex_chars(self, fake_superdesk):
        field = "field_name"
        value = "foo(bar++^$$^"  # an invalid regex pattern

        try:
            self.validator.validate({field: value}, {field: {"iunique": True}})
        except Exception as ex:
            self.fail("Error unexpectedly raised: {}".format(ex))


@mock.patch("superdesk.validator.superdesk")
class ValidateIuniquePerParentMethodTestCase(SuperdeskValidatorTest):
    """Tests for the _validate_iunique_per_parent() method."""

    def setUp(self):
        klass = self._get_target_class()
        self.validator = klass(schema={})
        self.validator.document = {}

    def test_does_not_raise_error_on_inputs_containing_special_regex_chars(self, fake_superdesk):
        parent_field = "parent_field_name"
        field = "field_name"
        value = "foo(bar++^$$^"  # an invalid regex pattern

        try:
            self.validator.validate({field: value}, {field: {"iunique_per_parent": parent_field}})
        except Exception as ex:
            self.fail("Error unexpectedly raised: {}".format(ex))


@mock.patch("superdesk.validator.superdesk")
class ValidateMultipleEmailsTestCase(SuperdeskValidatorTest):
    """Tests for the _validate_mulitple_emails() method."""

    def setUp(self):
        klass = self._get_target_class()
        self.validator = klass(schema={})
        self.validator.document = {}

    def test_does_not_raise_error_on_inputs_containing_email(self, fake_superdesk):

        field = "field_name"
        value = "abc@abc.com"

        try:
            self.assertTrue(self.validator.validate({field: value}, {field: {"multiple_emails": True}}))
        except Exception as ex:
            self.fail("Error unexpectedly raised: {}".format(ex))

    def test_does_not_raise_error_on_inputs_containing_emails(self, fake_superdesk):
        field = "field_name"
        value = "abc@abc.com,test@abc.com"

        try:
            self.validator._validate_multiple_emails(True, field, value)
            self.assertEqual(len(self.validator.errors), 0)
        except Exception as ex:
            self.fail("Error unexpectedly raised: {}".format(ex))

    def test_raise_error_on_inputs_containing_emails(self, fake_superdesk):
        field = "field_name"
        value = "abc@abc.com,test"
        try:
            self.assertFalse(self.validator.validate({field: value}, {field: {"multiple_emails": True}}))
            self.assertDictEqual(self.validator.errors, {"field_name": {"pattern": 1}})
        except Exception as ex:
            self.fail("Error unexpectedly raised: {}".format(ex))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import json
from unittest.mock import patch

from apps.prepopulate.app_populate import AppPopulateCommand
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.vocabularies import VocabulariesService
from superdesk.errors import SuperdeskApiError


class VocabulariesPopulateTest(TestCase):
    def setUp(self):
        self.filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), "vocabularies.json")
        self.json_data = [
            {
                "_id": "categories",
                "unique_field": "qcode",
                "items": [
                    {"name": "National", "qcode": "A", "is_active": True},
                    {"name": "Domestic Sports", "qcode": "T", "is_active": False},
                ],
            },
            {
                "_id": "newsvalue",
                "items": [
                    {"name": "1", "value": "1", "is_active": True},
                    {"name": "2", "value": "2", "is_active": True},
                    {"name": "3", "value": "3", "is_active": False},
                ],
            },
        ]

        with open(self.filename, "w+") as file:
            json.dump(self.json_data, file)

    def test_populate_vocabularies(self):
        cmd = AppPopulateCommand()
        cmd.run(self.filename)
        service = get_resource_service("vocabularies")

        for item in self.json_data:
            data = service.find_one(_id=item["_id"], req=None)
            self.assertEqual(data["_id"], item["_id"])
            self.assertListEqual(data["items"], item["items"])

    def test_check_uniqueness(self):
        items = [
            {"name": "National", "qcode": "A", "is_active": True},
            {"name": "Domestic Sports", "qcode": "a", "is_active": True},
        ]
        with self.assertRaises(SuperdeskApiError):
            VocabulariesService()._check_uniqueness(items, "qcode")

    def test_check_uniqueness_active_only(self):
        items = [
            {"name": "National", "qcode": "A", "is_active": True},
            {"name": "Domestic Sports", "qcode": "A", "is_active": False},
        ]
        VocabulariesService()._check_uniqueness(items, "qcode")

    def test_check_value_of_unique_field(self):
        items = [{"name": "National", "is_active": True}, {"name": "Domestic Sports", "qcode": "A", "is_active": True}]
        with self.assertRaises(SuperdeskApiError):
            VocabulariesService()._check_uniqueness(items, "qcode")

    def test_get_rightsinfo(self):
        service = get_resource_service("vocabularies")
        vocab = {
            "_id": "rightsinfo",
            "items": [
                {
                    "is_active": True,
                    "name": "default",
                    "copyrightHolder": "default holder",
                    "copyrightNotice": "default notice",
                    "usageTerms": "default terms",
                },
                {
                    "is_active": True,
                    "name": "foo",
                    "copyrightHolder": "foo holder",
                    "copyrightNotice": "foo notice",
                    "usageTerms": "foo terms",
                },
            ],
        }

        with patch.object(service, "find_one", return_value=vocab):
            info = service.get_rightsinfo({})
            self.assertEqual("default holder", info["copyrightholder"])
            self.assertEqual("default notice", info["copyrightnotice"])
            self.assertEqual("default terms", info["usageterms"])
            info = service.get_rightsinfo({"source": "foo"})
            self.assertEqual("foo holder", info["copyrightholder"])
            self.assertEqual("foo notice", info["copyrightnotice"])
            self.assertEqual("foo terms", info["usageterms"])

    def test_get_locale_vocabulary(self):
        items = [
            {
                "is_active": True,
                "name": "FIXME1",
                "qcode": "f",
                "subject": "",
                "translations": {"name": {"fr": "FIXME1-fr", "es": "FIXME1-es"}},
            },
            {
                "is_active": True,
                "name": "FIXME2",
                "qcode": "f",
                "subject": "",
                "translations": {"name": {"fr": "FIXME2-fr", "es": "FIXME2-es"}},
            },
        ]
        result = VocabulariesService().get_locale_vocabulary(items, "fr")

        self.assertEqual(result[0]["name"], "FIXME1-fr")
        self.assertEqual(result[1]["name"], "FIXME2-fr")

    def tearDown(self):
        os.remove(self.filename)



import os
import io
import eve
import bson
import unittest
from unittest.mock import Mock
from superdesk.upload import bp, upload_url
from superdesk.datalayer import SuperdeskDataLayer
from superdesk.storage import SuperdeskGridFSMediaStorage
from superdesk.utc import utcnow
from superdesk.utils import sha
from datetime import timedelta


class GridFSMediaStorageTestCase(unittest.TestCase):
    def setUp(self):
        self.app = eve.Eve(__name__, {"DOMAIN": {}})
        self.app.config["MEDIA_PREFIX"] = "http://localhost/upload-raw"
        self.app.config["DOMAIN"] = {"upload": {}}
        self.app.config["MONGO_DBNAME"] = "sptests"
        self.app.data = SuperdeskDataLayer(self.app)
        self.media = SuperdeskGridFSMediaStorage(self.app)
        self.app.register_blueprint(bp)
        self.app.upload_url = upload_url

    def test_url_for_media(self):
        _id = bson.ObjectId(sha("test")[:24])
        with self.app.app_context():
            url = self.media.url_for_media(_id)
        self.assertEqual("http://localhost/upload-raw/%s" % _id, url)

    def test_url_for_media_content_type(self):
        _id_str = "1" * 24
        _id = bson.ObjectId(_id_str)
        with self.app.app_context():
            url = self.media.url_for_media(_id, "image/jpeg")
        self.assertEqual("http://localhost/upload-raw/{}.jpg".format(_id_str), url)

    def test_put_media_with_id(self):
        data = io.StringIO("test data")
        filename = "x"

        gridfs = self._mock_gridfs()
        _id = bson.ObjectId()

        with self.app.app_context():
            self.media.put(data, filename, "text/plain", _id=str(_id))

        kwargs = {
            "content_type": "text/plain",
            "filename": filename,
            "metadata": None,
            "_id": _id,
        }

        gridfs.put.assert_called_once_with(data, **kwargs)

    def test_put_into_folder(self):
        data = b"test data"
        filename = "x"
        folder = "gridtest"

        gridfs = self._mock_gridfs()

        with self.app.app_context():
            self.media.put(data, filename=filename, content_type="text/plain", folder=folder)

        kwargs = {"content_type": "text/plain", "filename": "{}/{}".format(folder, filename), "metadata": None}

        gridfs.put.assert_called_once_with(data, **kwargs)

    def test_find_files(self):
        gridfs = self._mock_gridfs()
        upload_date = {"$lte": utcnow(), "$gte": utcnow() - timedelta(hours=1)}
        folder = "gridtest"
        query_filename = {"filename": {"$regex": "^{}/".format(folder)}}
        query_upload_date = {"uploadDate": upload_date}

        with self.app.app_context():
            self.media.find(folder=folder, upload_date=upload_date)
            gridfs.find.assert_called_once_with({"$and": [query_filename, query_upload_date]})

            self.media.find(folder=folder)
            gridfs.find.assert_called_with(query_filename)

            self.media.find(upload_date=upload_date)
            gridfs.find.assert_called_with(query_upload_date)

            self.media.find()
            gridfs.find.assert_called_with({})

    def test_custom_id(self):
        data = b"foo"
        with self.app.app_context():
            self.media.put(data, _id="foo")
            _file = self.media.get("foo")
            assert data == _file.read()

    def _mock_gridfs(self):
        gridfs = Mock()
        gridfs.put = Mock(return_value="y")
        gridfs.find = Mock(return_value=[])
        self.media._fs["MONGO"] = gridfs
        return gridfs

    def test_mimetype_detect(self):
        # keep default mimetype
        gridfs = self._mock_gridfs()
        _id = bson.ObjectId()
        content = b"bytes are here"
        filename = "extensionless"
        content_type = "text/css"
        with self.app.app_context():
            self.media.put(content, filename, content_type, _id=str(_id))
        kwargs = {
            "content_type": content_type,
            "filename": filename,
            "metadata": None,
            "_id": _id,
        }
        gridfs.put.assert_called_once_with(content, **kwargs)

        # get mimetype from the filename
        gridfs = self._mock_gridfs()
        _id = bson.ObjectId()
        content = b"bytes are here"
        filename = "styles.css"
        content_type = "application/pdf"
        with self.app.app_context():
            self.media.put(content, filename, content_type, _id=str(_id))
        kwargs = {
            "content_type": "text/css",
            "filename": filename,
            "metadata": None,
            "_id": _id,
        }
        gridfs.put.assert_called_once_with(content, **kwargs)

        gridfs = self._mock_gridfs()
        _id = bson.ObjectId()
        content = b"bytes are here"
        filename = "styles.JpG"
        content_type = "application/pdf"
        with self.app.app_context():
            self.media.put(content, filename, content_type, _id=str(_id))
        kwargs = {
            "content_type": "image/jpeg",
            "filename": filename,
            "metadata": None,
            "_id": _id,
        }
        gridfs.put.assert_called_once_with(content, **kwargs)

        # get mimetype from the file
        fixtures_path = os.path.join(os.path.dirname(__file__), "fixtures")
        with open(os.path.join(fixtures_path, "file_example-jpg.jpg"), "rb") as content:
            gridfs = self._mock_gridfs()
            _id = bson.ObjectId()
            filename = "extensionless"
            content_type = "dummy/text"
            with self.app.app_context():
                self.media.put(content, filename, content_type, _id=str(_id))
            kwargs = {
                "content_type": "image/jpeg",
                "filename": filename,
                "metadata": None,
                "_id": _id,
            }
            gridfs.put.assert_called_once_with(content, **kwargs)

        with open(os.path.join(fixtures_path, "file_example-xls.xls"), "rb") as content:
            gridfs = self._mock_gridfs()
            _id = bson.ObjectId()
            filename = "extensionless"
            content_type = "dummy/text"
            with self.app.app_context():
                self.media.put(content, filename, content_type, _id=str(_id))
            kwargs = {
                "content_type": "application/vnd.ms-excel",
                "filename": filename,
                "metadata": None,
                "_id": _id,
            }
            gridfs.put.assert_called_once_with(content, **kwargs)

        with open(os.path.join(fixtures_path, "file_example-xlsx.xlsx"), "rb") as content:
            gridfs = self._mock_gridfs()
            _id = bson.ObjectId()
            filename = "extensionless"
            content_type = "dummy/text"
            with self.app.app_context():
                self.media.put(content, filename, content_type, _id=str(_id))
            kwargs = {
                "content_type": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                "filename": filename,
                "metadata": None,
                "_id": _id,
            }
            gridfs.put.assert_called_once_with(content, **kwargs)

        with open(os.path.join(fixtures_path, "file_example-docx.docx"), "rb") as content:
            gridfs = self._mock_gridfs()
            _id = bson.ObjectId()
            filename = "extensionless"
            content_type = "dummy/text"
            with self.app.app_context():
                self.media.put(content, filename, content_type, _id=str(_id))
            kwargs = {
                "content_type": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                "filename": filename,
                "metadata": None,
                "_id": _id,
            }
            gridfs.put.assert_called_once_with(content, **kwargs)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from typing import Dict, Any
import logging
import superdesk
from eve.utils import config


log = logging.getLogger(__name__)

# elastic mapping helpers
not_indexed = {"type": "string", "index": "no"}  # noqa
not_analyzed = {"type": "string", "index": "not_analyzed"}
not_enabled = {"type": "object", "enabled": False}  # noqa
text_with_keyword = {
    "type": "text",
    "fields": {
        "keyword": {"type": "keyword"},
    },
}


def build_custom_hateoas(hateoas, doc, **values):
    values.update(doc)
    links = doc.get(config.LINKS)
    if not links:
        links = {}
        doc[config.LINKS] = links

    for link_name in hateoas.keys():
        link = hateoas[link_name]
        link = {"title": link["title"], "href": link["href"]}
        link["href"] = link["href"].format(**values)
        links[link_name] = link


class Resource:
    """
    Base model for all endpoints, defines the basic implementation for CRUD datalayer functionality.
    """

    endpoint_name = None
    url = None
    item_url = None
    additional_lookup = None
    schema = {}
    allow_unknown = None
    item_methods = None
    resource_methods = None
    public_methods = None
    public_item_methods = None
    extra_response_fields = None
    embedded_fields = None
    datasource: Dict[str, Any] = {}
    versioning = None
    internal_resource = None
    resource_title = None
    service = None
    endpoint_schema = None
    resource_preferences = None
    etag_ignore_fields = []
    mongo_prefix = None
    mongo_indexes = None
    auth_field = None
    authentication = None
    elastic_prefix = None
    query_objectid_as_string = None
    soft_delete = None
    hateoas = None
    merge_nested_documents = None
    projection = None

    def __init__(self, endpoint_name, app, service, endpoint_schema=None):
        self.endpoint_name = endpoint_name
        self.service = service
        if not endpoint_schema:
            endpoint_schema = {"schema": self.schema}
            if self.allow_unknown is not None:
                endpoint_schema.update({"allow_unknown": self.allow_unknown})
            if self.additional_lookup is not None:
                endpoint_schema.update({"additional_lookup": self.additional_lookup})
            if self.extra_response_fields is not None:
                endpoint_schema.update({"extra_response_fields": self.extra_response_fields})
            if self.datasource:
                endpoint_schema.update({"datasource": self.datasource})
            if self.item_methods is not None:
                endpoint_schema.update({"item_methods": self.item_methods})
            if self.resource_methods is not None:
                endpoint_schema.update({"resource_methods": self.resource_methods})
            if self.public_methods is not None:
                endpoint_schema.update({"public_methods": self.public_methods})
            if self.public_item_methods is not None:
                endpoint_schema.update({"public_item_methods": self.public_item_methods})
            if self.url is not None:
                endpoint_schema.update({"url": self.url})
            if self.item_url is not None:
                endpoint_schema.update({"item_url": self.item_url})
            if self.embedded_fields is not None:
                endpoint_schema.update({"embedded_fields": self.embedded_fields})
            if self.versioning is not None:
                endpoint_schema.update({"versioning": self.versioning})
            if self.internal_resource is not None:
                endpoint_schema.update({"internal_resource": self.internal_resource})
            if self.resource_title is not None:
                endpoint_schema.update({"resource_title": self.resource_title})
            if self.etag_ignore_fields:
                endpoint_schema.update({"etag_ignore_fields": self.etag_ignore_fields})
            if self.mongo_prefix:
                endpoint_schema.update({"mongo_prefix": self.mongo_prefix})
            if self.auth_field:
                endpoint_schema.update({"auth_field": self.auth_field})
            if self.authentication:
                endpoint_schema.update({"authentication": self.authentication})
            if self.elastic_prefix:
                endpoint_schema.update({"elastic_prefix": self.elastic_prefix})
            if self.query_objectid_as_string:
                endpoint_schema.update({"query_objectid_as_string": self.query_objectid_as_string})
            if self.soft_delete:
                endpoint_schema.update({"soft_delete": self.soft_delete})
            if self.hateoas is not None:
                endpoint_schema.update({"hateoas": self.hateoas})
            if self.merge_nested_documents is not None:
                endpoint_schema.update({"merge_nested_documents": self.merge_nested_documents})
            if self.mongo_indexes:
                # used in app:initialize_data
                endpoint_schema["mongo_indexes__init"] = self.mongo_indexes
            if self.projection is not None:
                endpoint_schema.update({"projection": self.projection})

            if app.config.get("SCHEMA_UPDATE", {}).get(self.endpoint_name):
                schema_updates = app.config["SCHEMA_UPDATE"][self.endpoint_name]
                log.warning(
                    "Updating {} schema with custom data for fields: {}".format(
                        self.endpoint_name.upper(), ", ".join(schema_updates.keys())
                    )
                )
                self.schema.update(schema_updates)

        self.endpoint_schema = endpoint_schema

        on_fetched_resource = getattr(app, "on_fetched_resource_%s" % self.endpoint_name)
        on_fetched_resource -= service.on_fetched
        on_fetched_resource += service.on_fetched

        on_fetched_item = getattr(app, "on_fetched_item_%s" % self.endpoint_name)
        on_fetched_item -= service.on_fetched_item
        on_fetched_item += service.on_fetched_item

        on_insert_event = getattr(app, "on_insert_%s" % self.endpoint_name)
        on_insert_event -= service.on_create
        on_insert_event += service.on_create

        on_inserted_event = getattr(app, "on_inserted_%s" % self.endpoint_name)
        on_inserted_event -= service.on_created
        on_inserted_event += service.on_created

        on_update_event = getattr(app, "on_update_%s" % self.endpoint_name)
        on_update_event -= service.on_update
        on_update_event += service.on_update

        on_updated_event = getattr(app, "on_updated_%s" % self.endpoint_name)
        on_updated_event -= service.on_updated
        on_updated_event += service.on_updated

        on_replace_event = getattr(app, "on_replace_%s" % self.endpoint_name)
        on_replace_event -= service.on_replace
        on_replace_event += service.on_replace

        on_replaced_event = getattr(app, "on_replaced_%s" % self.endpoint_name)
        on_replaced_event -= service.on_replaced
        on_replaced_event += service.on_replaced

        on_delete_event = getattr(app, "on_delete_item_%s" % self.endpoint_name)
        on_delete_event -= service.on_delete
        on_delete_event += service.on_delete

        on_deleted_event = getattr(app, "on_deleted_item_%s" % self.endpoint_name)
        on_deleted_event -= service.on_deleted
        on_deleted_event += service.on_deleted

        app.register_resource(self.endpoint_name, endpoint_schema)
        superdesk.resources[self.endpoint_name] = self

        for request_method in ["GET", "POST", "PATCH", "PUT", "DELETE"]:
            if hasattr(self, "pre_request_" + request_method.lower()):
                hook_event_name = "on_pre_" + request_method + "_" + self.endpoint_name
                hook_event = getattr(app, hook_event_name)
                hook_method = getattr(self, "pre_request_" + request_method.lower())
                hook_event -= hook_method
                hook_event += hook_method

        # Register callbacks for operations on other resources
        # The callback format is: on_<operation>_res_<other_resource>
        # where <operation> can be: fetched, fetched_item, create, created, update
        # updated, delete, deleted
        operations_events = {
            "fetched": "fetched_resource",
            "fetched_item": "fetched_item",
            "create": "insert",
            "created": "inserted",
            "update": "update",
            "updated": "updated",
            "delete": "delete_item",
            "deleted": "deleted_item",
        }
        for method in [method for method in dir(service) if method.startswith("on_")]:
            for operation, eve_event in operations_events.items():
                method_prefix = "on_%s_res_" % operation
                service_method = getattr(service, method)
                if method.startswith(method_prefix) and callable(service_method):
                    foreign_endpoint_name = method[len(method_prefix) :]
                    if foreign_endpoint_name not in superdesk.resources:
                        raise RuntimeError('Invalid hook "%s" in service "%s"' % (method, type(service)))
                    eve_hook = getattr(app, "on_%s_%s" % (eve_event, foreign_endpoint_name))
                    eve_hook -= service_method
                    eve_hook += service_method

    @staticmethod
    def rel(resource, embeddable=True, required=False, type="objectid", nullable=False, readonly=False):
        return {
            "type": type,
            "required": required,
            "nullable": nullable,
            "readonly": readonly,
            "data_relation": {"resource": resource, "field": "_id", "embeddable": embeddable},
            "mapping": {"type": "keyword"},
        }

    @staticmethod
    def int(required=False, nullable=False):
        return {
            "type": "integer",
            "required": required,
            "nullable": nullable,
        }

    @staticmethod
    def not_analyzed_field(type="string"):
        return {
            "type": type,
            "mapping": not_analyzed,
        }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


LINKED_IN_PACKAGES = "linked_in_packages"
PACKAGE = "package"
PACKAGE_TYPE = "package_type"
REFS = "refs"
RESIDREF = "residRef"
ID_REF = "idRef"
ROLE = "role"
GROUPS = "groups"
MAIN_GROUP = "main"
MAIN_ROLE = "grpRole:main"
ROOT_GROUP = "root"
ROOT_ROLE = "grpRole:NEP"
GROUP_ID = "id"



import logging
import re
from superdesk.services import BaseService
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError
from superdesk.publish.formatters import get_all_formatters
from superdesk.utils import get_random_string
from superdesk.validation import ValidationError
from io import BytesIO
from zipfile import ZipFile
from flask import current_app as app
from flask_babel import _

logger = logging.getLogger(__name__)


class ExportService(BaseService):
    def create(self, docs, **kwargs):
        doc = docs[0]
        formatter = self._validate_and_get_formatter(doc)

        validate = doc.get("validate", False)
        archive_service = get_resource_service("archive")

        unsuccessful_exports = 0
        try:
            in_memory_zip = BytesIO()
            with ZipFile(in_memory_zip, "a") as zip:
                for item_id in doc.get("item_ids"):
                    item = archive_service.find_one(req=None, _id=item_id)
                    if item:
                        try:
                            if validate:
                                self._validate_for_publish(item)

                            contents = formatter.export(item)
                            # Remove invalid filename chars (for windows OS) and create the filename
                            filename = (
                                re.sub(r'[\\/*?:"<>|]', "", item.get("slugline", ""))
                                + "_"
                                + str(item.get("unique_id"))
                                + ".txt"
                            )
                            zip.writestr(filename, contents.encode("UTF-8"))
                        except ValidationError:
                            unsuccessful_exports += 1
                    else:
                        unsuccessful_exports += 1

            url = None
            # Store the zip file on media_storage
            # only if at least one item is formatted successfully
            if unsuccessful_exports < len(doc.get("item_ids")):
                zip_id = app.media.put(
                    in_memory_zip.getvalue(),
                    filename="export_{}.zip".format(get_random_string()),
                    content_type="application/zip",
                    folder="temp",
                )
                url = app.media.url_for_download(zip_id, "application/zip")

            doc["url"] = url
            doc["failures"] = unsuccessful_exports
            return [len(docs)]
        except Exception as ex:
            raise SuperdeskApiError.badRequestError(
                _("Error creating export zip file. Try again please."), exception=ex
            )

    def _validate_for_publish(self, doc):
        """Validates the given story for publish action"""
        validate_item = {"act": "publish", "type": doc["type"], "validate": doc}
        validation_errors = get_resource_service("validate").validate(validate_item)
        if validation_errors:
            raise ValidationError(validation_errors)

    def _validate_and_get_formatter(self, doc):
        """Validates incoming request and gets the formatter to be used"""
        if doc.get("item_ids") == 0:
            raise SuperdeskApiError.badRequestError(_("No items to export."))

        formatter_name = doc.get("format_type")
        formatter = next((f for f in get_all_formatters() if type(f).__name__ == formatter_name), None)
        if not formatter:
            raise SuperdeskApiError.badRequestError(_("Formatter not found for requested format type."))

        return formatter



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""NewsML G2 Superdesk formatter"""

import superdesk

from lxml import etree
from lxml.etree import SubElement
from flask import current_app as app

from superdesk import text_utils
from superdesk.publish.formatters import Formatter
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO, FORMATS, FORMAT
from superdesk.utc import utcnow
from superdesk.errors import FormatterError
from superdesk.publish.formatters.nitf_formatter import NITFFormatter
from superdesk.metadata.packages import REFS, RESIDREF, ROLE, GROUPS, GROUP_ID, ID_REF
from superdesk.filemeta import get_filemeta
from superdesk import etree as sd_etree
from superdesk.geonames import get_geonames_country_qcode, get_geonames_state_qcode, get_geonames_qcode
from apps.archive.common import ARCHIVE, get_utc_schedule

XML_LANG = "{http://www.w3.org/XML/1998/namespace}lang"


def get_newsml_provider_id():
    return app.config.get("NEWSML_PROVIDER_ID")


def _get_cv_qcode(item):
    if item.get("qcode"):
        return item["qcode"]
    if item.get("code"):
        return ":".join([piece for piece in [item.get("scheme"), item["code"]] if piece])
    return item["name"]


class NewsMLG2Formatter(Formatter):
    """NewsML G2 Formatter"""

    ENCODING = "UTF-8"
    XML_ROOT = '<?xml version="1.0" encoding="{}"?>'.format(ENCODING)

    _message_nsmap = {
        None: "http://iptc.org/std/nar/2006-10-01/",
        "x": "http://www.w3.org/1999/xhtml",
        "xsi": "http://www.w3.org/2001/XMLSchema-instance",
    }

    _debug_message_extra = {
        "{{{}}}schemaLocation".format(
            _message_nsmap["xsi"]
        ): "http://iptc.org/std/nar/2006-10-01/ \
    http://www.iptc.org/std/NewsML-G2/2.18/specification/NewsML-G2_2.18-spec-All-Power.xsd"
    }

    def _format_date(self, date):
        return date.strftime("%Y-%m-%dT%H:%M:%S+00:00")

    def format(self, article, subscriber, codes=None):
        """Create article in NewsML G2 format

        :param dict article:
        :param dict subscriber:
        :param list codes: selector codes
        :return [(int, str)]: return a List of tuples. A tuple consist of
            publish sequence number and formatted article string.
        :raises FormatterError: if the formatter fails to format an article
        """
        try:
            self.subscriber = subscriber
            pub_seq_num = superdesk.get_resource_service("subscribers").generate_sequence_number(subscriber)
            is_package = self._is_package(article)
            news_message = etree.Element("newsMessage", attrib=self._debug_message_extra, nsmap=self._message_nsmap)
            self._format_header(article, news_message, pub_seq_num)
            item_set = self._format_item(news_message)
            if is_package:
                item = self._format_item_set(article, item_set, "packageItem")
                self._format_groupset(article, item)
            elif article[ITEM_TYPE] in {CONTENT_TYPE.PICTURE, CONTENT_TYPE.AUDIO, CONTENT_TYPE.VIDEO}:
                item = self._format_item_set(article, item_set, "newsItem")
                self._format_contentset(article, item)
            else:
                nitfFormater = NITFFormatter()
                nitf = nitfFormater.get_nitf(article, subscriber, pub_seq_num)
                newsItem = self._format_item_set(article, item_set, "newsItem")
                self._format_content(article, newsItem, nitf)

            sd_etree.fix_html_void_elements(news_message)
            return [
                (
                    pub_seq_num,
                    self.XML_ROOT
                    + etree.tostring(news_message, pretty_print=True, encoding=self.ENCODING).decode(self.ENCODING),
                )
            ]
        except Exception as ex:
            raise FormatterError.newmsmlG2FormatterError(ex, subscriber)

    def _is_package(self, article):
        """Given an article returns if it is a none takes package or not

        :param artcile:
        :return: True is package
        """
        return article[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE

    def _format_header(self, article, news_message, pub_seq_num):
        """Creates the header element of the newsMessage.

        :param dict article:
        :param Element news_message:
        :param int pub_seq_num:
        """
        header = SubElement(news_message, "header")
        SubElement(header, "sent").text = self._format_date(utcnow())
        SubElement(header, "sender").text = get_newsml_provider_id()
        SubElement(header, "transmitId").text = str(pub_seq_num)
        SubElement(header, "priority").text = str(article.get("priority", 5))
        SubElement(header, "origin").text = article.get("original_source", article.get("source", ""))

    def _format_item(self, news_message):
        return SubElement(news_message, "itemSet")

    def _format_item_set(self, article, item_set, item_type):
        """Construct the item element (newsItem or packageItem) and append the item_meta and contentMeta entities

        :param dict article:
        :param element item_set:
        :param str item_type:
        """
        item = SubElement(
            item_set,
            item_type,
            attrib={
                "standard": "NewsML-G2",
                "standardversion": "2.18",
                "guid": article["guid"],
                "version": str(article[superdesk.config.VERSION]),
                XML_LANG: self._get_lang(article),
                "conformance": "power",
            },
        )
        SubElement(
            item, "catalogRef", attrib={"href": "http://www.iptc.org/std/catalog/catalog.IPTC-G2-Standards_25.xml"}
        )
        self._format_rights(item, article)

        item_meta = SubElement(item, "itemMeta")
        self._format_item_meta(article, item_meta, item)

        content_meta = SubElement(item, "contentMeta")
        self._format_content_meta(article, content_meta, item)

        if article[ITEM_TYPE] in {CONTENT_TYPE.PICTURE, CONTENT_TYPE.AUDIO, CONTENT_TYPE.VIDEO}:
            self._format_description(article, content_meta)
            self._format_creditline(article, content_meta)

        return item

    def _format_item_meta(self, article, item_meta, item):
        self._format_itemClass(article, item_meta)
        self._format_provider(item_meta)
        self._format_versioncreated(article, item_meta)
        self._format_firstcreated(article, item_meta)
        self._format_pubstatus(article, item_meta)

        if article.get(EMBARGO):
            SubElement(item_meta, "embargoed").text = get_utc_schedule(article, EMBARGO).isoformat()

        # optional properties
        self._format_ednote(article, item_meta)
        self._format_signal(article, item_meta)

    def _format_content_meta(self, article, content_meta, item):
        SubElement(content_meta, "urgency").text = str(article.get("urgency", 5))
        self._format_timestamps(article, content_meta)
        self._format_creator(article, content_meta)
        self._format_located(article, content_meta)
        self._format_subject(article, content_meta)
        self._format_genre(article, content_meta)
        self._format_slugline(article, content_meta)
        self._format_headline(article, content_meta)
        self._format_place(article, content_meta)
        self._format_category(article, content_meta)
        self._format_company_codes(article, content_meta, item)

    def _format_content(self, article, news_item, nitf):
        """Adds the content set to the xml

        :param dict article:
        :param Element newsItem:
        :param Element nitf:
        """
        content_set = SubElement(news_item, "contentSet")
        if article.get(FORMAT) == FORMATS.PRESERVED:
            inline_data = text_utils.get_text(self.append_body_footer(article))
            SubElement(content_set, "inlineData", attrib={"contenttype": "text/plain"}).text = inline_data
        elif article[ITEM_TYPE] in [CONTENT_TYPE.TEXT, CONTENT_TYPE.COMPOSITE]:
            inline = SubElement(content_set, "inlineXML", attrib={"contenttype": "application/nitf+xml"})
            inline.append(nitf)

    def _format_rights(self, newsItem, article):
        """Adds the rightsholder section to the newsItem

        :param Element newsItem:
        :param dict article:
        """
        rights = superdesk.get_resource_service("vocabularies").get_rightsinfo(article)
        rightsinfo = SubElement(newsItem, "rightsInfo")
        holder = SubElement(rightsinfo, "copyrightHolder")
        SubElement(holder, "name").text = rights["copyrightholder"]
        SubElement(rightsinfo, "copyrightNotice").text = rights["copyrightnotice"]
        SubElement(rightsinfo, "usageTerms").text = rights["usageterms"]

    # itemClass elements
    def _format_itemClass(self, article, item_meta):
        """Append the item class to the item_meta data element

        :param dict article:
        :param Element item_meta:
        """
        if CONTENT_TYPE.COMPOSITE and self._is_package(article):
            SubElement(item_meta, "itemClass", attrib={"qcode": "ninat:composite"})
            return
        if article[ITEM_TYPE] in {CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED, CONTENT_TYPE.COMPOSITE}:
            SubElement(item_meta, "itemClass", attrib={"qcode": "ninat:text"})
        elif article[ITEM_TYPE] in {CONTENT_TYPE.PICTURE, CONTENT_TYPE.AUDIO, CONTENT_TYPE.VIDEO}:
            SubElement(item_meta, "itemClass", attrib={"qcode": "ninat:%s" % article[ITEM_TYPE].lower()})

    def _format_provider(self, item_meta):
        """Appends the provider element to the item_meta element

        :param dict article:
        :param Element item_meta:
        """
        provider = SubElement(item_meta, "provider")
        SubElement(provider, "name").text = get_newsml_provider_id()

    def _format_versioncreated(self, article, item_meta):
        """Appends the versionCreated element to the item_meta element.

        :param dict article:
        :param Element item_meta:
        """
        SubElement(item_meta, "versionCreated").text = self._format_date(article["versioncreated"])

    def _format_firstcreated(self, article, item_meta):
        """Appends the firstCreated element to the item_meta element.

        :param dict article:
        :param Element item_meta:
        """
        SubElement(item_meta, "firstCreated").text = self._format_date(article["firstcreated"])

    def _format_pubstatus(self, article, item_meta):
        """Appends the pubStatus element to the item_meta element.

        :param dict article:
        :param Element item_meta:
        """
        SubElement(item_meta, "pubStatus", attrib={"qcode": "stat:" + article.get("pubstatus", "usable")})

    def _format_signal(self, article, item_meta):
        """Appends the signal element to the item_meta element.

        :param dict article:
        :param Element item_meta:
        """
        if article["state"] == "Corrected":
            SubElement(item_meta, "signal", attrib={"qcode": "sig:correction"})
        else:
            SubElement(item_meta, "signal", attrib={"qcode": "sig:update"})

    def _format_ednote(self, article, item_meta):
        """Appends the edNote element to the item_meta element.

        :param dict article:
        :param Element item_meta:
        """
        if "ednote" in article and article.get("ednote", "") != "":
            SubElement(item_meta, "edNote").text = article.get("ednote", "")

    # contentMeta elements
    def _format_timestamps(self, article, content_meta):
        """Appends the contentCreated and contentModified element to the contentMeta element.

        :param dict article:
        :param Element content_meta:
        """
        SubElement(content_meta, "contentCreated").text = self._format_date(article["firstcreated"])
        SubElement(content_meta, "contentModified").text = self._format_date(article["versioncreated"])

    def _format_creator(self, article, content_meta):
        """Appends the creator element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        if "byline" in article:
            creator = SubElement(content_meta, "creator")
            SubElement(creator, "name").text = article.get("byline", "") or ""

    def _format_subject(self, article, content_meta):
        """Appends the subject element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        if "subject" in article and len(article["subject"]) > 0:
            for s in article["subject"]:
                if "qcode" in s:
                    subj = SubElement(
                        content_meta, "subject", attrib={"type": "cpnat:abstract", "qcode": "subj:" + s["qcode"]}
                    )
                    self._format_translated_name(subj, s, article)

    def _format_genre(self, article, content_meta):
        """Appends the genre element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        if article.get("genre"):
            for g in article["genre"]:
                if g.get("name"):
                    genre = SubElement(content_meta, "genre", attrib={"qcode": _get_cv_qcode(g)})
                    self._format_translated_name(genre, g, article)

    def _format_category(self, article, content_meta):
        """Appends the subject element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        for category in article.get("anpa_category", []):
            subject = SubElement(
                content_meta, "subject", attrib={"type": "cpnat:abstract", "qcode": "cat:" + category["qcode"]}
            )
            self._format_translated_name(subject, category, article)

    def _format_slugline(self, article, content_meta):
        """Appends the slugline element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        SubElement(content_meta, "slugline").text = article.get("slugline", "")

    def _format_headline(self, article, content_meta):
        """Appends the headline element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        SubElement(content_meta, "headline").text = article.get("headline", "")

    def _format_place(self, article, content_meta):
        """Appends the subject (of type geoArea) element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        if not article.get("place"):
            return

        for place in article.get("place", []):
            if place.get("scheme") == "geonames":
                self._format_geonames_place(place, content_meta)
            elif place.get("state"):
                subject = self._create_subject_element(content_meta, place.get("state"), "loctyp:CountryArea")
                self._create_broader_element(subject, place.get("country"), "loctyp:Country")
                self._create_broader_element(subject, place.get("world_region"), "loctyp:WorldArea")
            elif place.get("country"):
                subject = self._create_subject_element(content_meta, place.get("country"), "loctyp:Country")
                self._create_broader_element(subject, place.get("world_region"), "loctyp:WorldArea")
            elif place.get("world_region"):
                self._create_subject_element(content_meta, place.get("world_region"), "loctyp:WorldArea")

    def _format_geonames_place(self, place, content_meta):
        subject = self._create_subject_element(
            content_meta, place.get("name", ""), get_geonames_qcode(place), "cpnat:geoArea"
        )
        if place.get("state") and place.get("feature_class", "").upper() != "A":
            self._create_broader_element(
                subject, place.get("state"), get_geonames_state_qcode(place), "cptype:statprov"
            )
        if place.get("state") or place.get("feature_class", "").upper() != "A":
            self._create_broader_element(
                subject, place.get("country"), get_geonames_country_qcode(place), "cptype:country"
            )
        location = place.get("location")
        if location:
            geo_area_details = SubElement(subject, "geoAreaDetails")
            SubElement(
                geo_area_details,
                "position",
                attrib={
                    "latitude": str(location.get("lat")),
                    "longitude": str(location.get("lon")),
                },
            )

    def _create_broader_element(self, parent, broader_name, qcode, concept_type="cpnat:abstract"):
        """Create broader element.

        :param element parent: parent element under which the broader element is created
        :param str broader_name: value for the name element
        :param str qcode:
        :param str concept_type:
        """
        if broader_name:
            broader_elm = SubElement(parent, "broader", attrib={"type": concept_type, "qcode": qcode})
            SubElement(broader_elm, "name").text = broader_name

    def _create_subject_element(self, parent, subject_name, qcode, concept_type="cpnat:abstract"):
        """Create a subject element

        :param element parent:
        :param str subject_name: value for the name element
        :param str qcode
        :param str concept_type
        :return: returns the subject element.
        """
        subject_elm = SubElement(parent, "subject", attrib={"type": concept_type, "qcode": qcode})
        SubElement(subject_elm, "name").text = subject_name
        return subject_elm

    def _format_located(self, article, content_meta):
        """Appends the located element to the contentMeta element

        :param dict article:
        :param Element content_meta:
        """
        located = article.get("dateline", {}).get("located", {})
        if located and located.get("city"):
            located_elm = SubElement(content_meta, "located", attrib={"type": "cpnat:abstract", "qcode": "loctyp:City"})
            SubElement(located_elm, "name").text = located.get("city")
            self._create_broader_element(located_elm, located.get("state"), "loctyp:CountryArea")
            self._create_broader_element(located_elm, located.get("country"), "loctyp:Country")

        if article.get("dateline", {}).get("text", {}):
            SubElement(content_meta, "dateline").text = article.get("dateline", {}).get("text", {})

    def _format_description(self, article, content_meta):
        """Appends the image description to the contentMeta element

        :param article:
        :param content_meta:
        """
        text = article.get("description_text", article.get("description", ""))
        SubElement(content_meta, "description", attrib={"role": "drol:caption"}).text = text

    def _format_creditline(self, article, content_meta):
        """Append the creditLine to the contentMeta for a picture

        :param article:
        :param content_meta:
        """
        SubElement(content_meta, "creditline").text = article.get("original_source", article.get("source", ""))

    def _format_groupset(self, article, item):
        """Constructs the groupSet element of a packageItem

        :param article:
        :param item:
        :return: groupSet appended to the item
        """
        groupSet = SubElement(item, "groupSet", attrib={"root": "root"})
        for group in article.get(GROUPS, []):
            attrib = {"id": group.get(GROUP_ID), "role": group.get(ROLE, "grpRole:%s" % group.get(GROUP_ID))}
            group_Elem = SubElement(groupSet, "group", attrib=attrib)
            for ref in group.get(REFS, []):
                if ID_REF in ref:
                    SubElement(group_Elem, "groupRef", attrib={"idref": ref.get(ID_REF)})
                else:
                    if RESIDREF in ref:
                        # get the current archive item being refered to
                        archive_item = superdesk.get_resource_service(ARCHIVE).find_one(req=None, _id=ref.get(RESIDREF))
                        if archive_item:
                            self._format_itemref(group_Elem, ref, archive_item)

    def _format_itemref(self, group, ref, item):
        attrib = {"residref": ref.get(RESIDREF), "contenttype": "application/vnd.iptc.g2.newsitem+xml"}
        itemRef = SubElement(group, "itemRef", attrib=attrib)
        SubElement(itemRef, "itemClass", attrib={"qcode": "ninat:" + ref.get(ITEM_TYPE, "text")})
        self._format_pubstatus(item, itemRef)
        self._format_headline(item, itemRef)
        self._format_slugline(item, itemRef)
        return itemRef

    def _format_contentset(self, article, item):
        """Constructs the contentSet element in a picture, video and audio newsItem.

        :param article:
        :param item:
        :return: contentSet Element added to the item
        """
        content_set = SubElement(item, "contentSet")
        for rendition, value in article.get("renditions", {}).items():
            attrib = {
                "href": value.get("href"),
                "contenttype": value.get("mimetype", ""),
                "rendition": "rendition:" + rendition,
            }
            if article.get(ITEM_TYPE) == CONTENT_TYPE.PICTURE:
                if "height" in value:
                    attrib["height"] = str(value.get("height"))
                if "width" in value:
                    attrib["width"] = str(value.get("width"))
            elif article.get(ITEM_TYPE) in {CONTENT_TYPE.VIDEO, CONTENT_TYPE.AUDIO}:
                if get_filemeta(article, "width"):
                    attrib["width"] = str(get_filemeta(article, "width"))
                if get_filemeta(article, "height"):
                    attrib["height"] = str(get_filemeta(article, "height"))
                if get_filemeta(article, "duration"):
                    attrib["duration"] = get_filemeta(article, "duration")
                    attrib["durationunit"] = "timeunit:normalPlayTime"

            if rendition == "original" and get_filemeta(article, "length"):
                attrib["size"] = str(get_filemeta(article, "length"))
            SubElement(content_set, "remoteContent", attrib=attrib)

    def _format_company_codes(self, article, content_meta, item):
        """Format copy codes.

        For each company in the article, appends the subject element to the contentMeta element
        and assert element to item

        :param article: object having published article details
        :type article: dict
        :param content_meta: object representing <contentMeta> in the XML tree
        :type content_meta: lxml.etree.Element
        :param item: object representing <newsItem> in the XML tree
        :type item: lxml.etree.Element
        """

        for company in article.get("company_codes", []):
            literal_name = company["qcode"]
            subject = SubElement(
                content_meta, "subject", attrib={"type": "cpnat:organisation", "literal": literal_name}
            )
            SubElement(subject, "name").text = company.get("name", "")

            assert_element = SubElement(item, "assert", attrib={"literal": literal_name})
            org_details_element = SubElement(assert_element, "organisationDetails")
            SubElement(
                org_details_element,
                "hasInstrument",
                attrib={"symbol": company.get("qcode", ""), "marketlabel": company.get("security_exchange", "")},
            )

    def can_format(self, format_type, article):
        """Method check if the article can be formatted to NewsML G2 or not.

        :param str format_type:
        :param dict article:
        :return: True if article can formatted else False
        """
        return format_type == "newsmlg2" and article[ITEM_TYPE] in {
            CONTENT_TYPE.TEXT,
            CONTENT_TYPE.PREFORMATTED,
            CONTENT_TYPE.COMPOSITE,
            CONTENT_TYPE.PICTURE,
            CONTENT_TYPE.VIDEO,
            CONTENT_TYPE.AUDIO,
        }

    def _get_translated_name(self, subject, article):
        """Get translated name for cv item.

        First checks full lang id with possible country,
        then just language id, then uses name assuming
        it's in app default language.
        """
        lang = self._get_lang(article)
        translations = subject.get("translations") or {}
        try:
            return translations["name"][lang], lang
        except KeyError:
            pass
        try:
            lang = lang.replace("-", "_").split("_")[0]
            return translations["name"][lang], lang
        except KeyError:
            pass
        return subject.get("name", ""), app.config["DEFAULT_LANGUAGE"]

    def _format_translated_name(self, dest, subject, article):
        name, lang = self._get_translated_name(subject, article)
        SubElement(dest, "name", attrib={XML_LANG: lang}).text = name

    def _get_lang(self, article):
        return article.get("language", app.config["DEFAULT_LANGUAGE"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import json

from flask import request, current_app as app
from eve.utils import config
from eve.methods.common import serialize_value
from flask_babel import _

from superdesk import privilege, get_resource_service
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.users import get_user_from_request
from superdesk.utc import utcnow
from superdesk.errors import SuperdeskApiError
from superdesk.default_schema import DEFAULT_SCHEMA, DEFAULT_EDITOR
from copy import deepcopy

logger = logging.getLogger(__name__)

KEYWORDS_CV = "keywords"


privilege(name="vocabularies", label="Vocabularies Management", description="User can manage vocabularies' contents.")


# TODO(petr): add api to specify vocabulary schema
vocab_schema = {
    "crop_sizes": {
        "width": {"type": "integer"},
        "height": {"type": "integer"},
    }
}


class VocabulariesResource(Resource):
    schema = {
        "_id": {
            "type": "string",
            "unique": True,
            "required": True,
            "regex": "^[a-zA-Z0-9-_]+$",
        },
        "display_name": {"type": "string", "required": True},
        "description": {"type": "string", "required": False},
        "helper_text": {"type": "string", "maxlength": 120},
        "tags": {
            "type": "list",
            "required": False,
            "schema": {
                "type": "dict",
                "schema": {"text": {"type": "string"}},
            },
        },
        "popup_width": {"type": "integer", "nullable": True},
        "type": {
            "type": "string",
            "required": True,
            "allowed": ["manageable", "unmanageable"],
        },
        "items": {
            "type": "list",
            "required": True,
            "schema": {
                "type": "dict",
                "allow_unknown": True,
                "schema": {
                    "name": {"type": "string", "required": False, "nullable": True},
                },
            },
        },
        "selection_type": {
            "type": "string",
            "allowed": ["single selection", "multi selection", "do not show"],
            "nullable": True,
        },
        "schema_field": {"type": "string", "required": False, "nullable": True},
        "dependent": {
            "type": "boolean",
        },
        "service": {
            "type": "dict",
            "schema": {},
            "allow_unknown": True,
            "keysrules": {"type": "string"},
            "valuesrules": {"type": "integer"},
        },
        "priority": {"type": "integer"},
        "unique_field": {"type": "string", "required": False, "nullable": True},
        "schema": {
            "type": "dict",
            "schema": {},
            "allow_unknown": True,
        },
        "field_type": {
            "type": "string",
            "nullable": True,
        },
        "field_options": {
            "type": "dict",
            "schema": {},
            "allow_unknown": True,
        },
        "init_version": {
            "type": "integer",
        },
        "preffered_items": {
            "type": "boolean",
        },
        "disable_entire_category_selection": {"type": "boolean", "default": False},
        "date_shortcuts": {
            "type": "list",
            "nullable": True,
            "schema": {
                "type": "dict",
                "schema": {
                    "value": {"type": "integer", "required": True},
                    "term": {"type": "string", "required": True},
                    "label": {"type": "string", "required": True},
                },
            },
        },
        "custom_field_type": {
            "type": "string",
            "nullable": True,
        },
        "custom_field_config": {
            "type": "dict",
            "nullable": True,
            "schema": {
                "increment_steps": {
                    "type": "list",
                },
                "initial_offset_minutes": {
                    "type": "integer",
                },
            },
        },
        "translations": {
            "type": "dict",
            "schema": {
                "display_name": {
                    "type": "dict",
                },
            },
        },
    }

    soft_delete = True
    item_url = r'regex("[-_\w]+")'
    item_methods = ["GET", "PATCH", "DELETE"]
    resource_methods = ["GET", "POST"]
    privileges = {"PATCH": "vocabularies", "POST": "vocabularies", "DELETE": "vocabularies"}
    mongo_indexes = {"field_type": [("field_type", 1)]}


class VocabulariesService(BaseService):

    system_keys = set(DEFAULT_SCHEMA.keys()).union(set(DEFAULT_EDITOR.keys()))

    def _validate_items(self, update):
        # if we have qcode and not unique_field set, we want it to be qcode
        try:
            update["schema"]["qcode"]
        except KeyError:
            pass
        else:
            update.setdefault("unique_field", "qcode")
        unique_field = update.get("unique_field")
        vocabs = {}
        if "schema" in update and "items" in update:
            for index, item in enumerate(update["items"]):
                for field, desc in update.get("schema", {}).items():
                    if (desc.get("required", False) or unique_field == field) and (
                        field not in item or not item[field]
                    ):
                        msg = "Required " + field + " in item " + str(index)
                        payload = {"error": {"required_field": 1}, "params": {"field": field, "item": index}}
                        raise SuperdeskApiError.badRequestError(message=msg, payload=payload)

                    elif desc.get("link_vocab") and desc.get("link_field"):
                        if not vocabs.get(desc["link_vocab"]):
                            linked_vocab = self.find_one(req=None, _id=desc["link_vocab"]) or {}

                            vocabs[desc["link_vocab"]] = [
                                vocab.get(desc["link_field"]) for vocab in linked_vocab.get("items") or []
                            ]

                        if item.get(field) and item[field] not in vocabs[desc["link_vocab"]]:
                            msg = '{} "{}={}" not found'.format(desc["link_vocab"], desc["link_field"], item[field])
                            payload = {"error": {"required_field": 1, "params": {"field": field, "item": index}}}
                            raise SuperdeskApiError.badRequestError(message=msg, payload=payload)

    def on_create(self, docs):
        for doc in docs:
            self._validate_items(doc)

            if doc.get("field_type") and doc["_id"] in self.system_keys:
                raise SuperdeskApiError(message="{} is in use".format(doc["_id"]), payload={"_id": {"conflict": 1}})

            if self.find_one(req=None, **{"_id": doc["_id"], "_deleted": True}):
                raise SuperdeskApiError(
                    message="{} is used by deleted vocabulary".format(doc["_id"]), payload={"_id": {"deleted": 1}}
                )

    def on_created(self, docs):
        for doc in docs:
            self._send_notification(doc, event="vocabularies:created")

    def on_replace(self, document, original):
        self._validate_items(document)
        document[app.config["LAST_UPDATED"]] = utcnow()
        document[app.config["DATE_CREATED"]] = (
            original.get(app.config["DATE_CREATED"], utcnow()) if original else utcnow()
        )
        logger.info("updating vocabulary item: %s", document["_id"])

    def on_fetched(self, doc):
        """Overriding to filter out inactive vocabularies and pops out 'is_active' property from the response.

        It keeps it when requested for manageable vocabularies.
        """

        if request and hasattr(request, "args") and request.args.get("where"):
            where_clause = json.loads(request.args.get("where"))
            if where_clause.get("type") == "manageable":
                return doc

        for item in doc[config.ITEMS]:
            self._filter_inactive_vocabularies(item)
            self._cast_items(item)

    def on_fetched_item(self, doc):
        """
        Overriding to filter out inactive vocabularies and pops out 'is_active' property from the response.
        """
        self._filter_inactive_vocabularies(doc)
        self._cast_items(doc)

    def on_update(self, updates, original):
        """Checks the duplicates if a unique field is defined"""
        if "items" in updates:
            updated = deepcopy(original)
            updated.update(updates)
            self._validate_items(updated)
        unique_field = original.get("unique_field")
        if unique_field:
            self._check_uniqueness(updates.get("items", []), unique_field)

    def on_updated(self, updates, original):
        """
        Overriding this to send notification about the replacement
        """
        self._send_notification(original)

    def on_replaced(self, document, original):
        """
        Overriding this to send notification about the replacement
        """
        self._send_notification(document)

    def on_delete(self, doc):
        """
        Overriding to validate vocabulary deletion
        """
        if "field_type" not in doc:
            raise SuperdeskApiError.badRequestError("Default vocabularies cannot be deleted")

    def _check_uniqueness(self, items, unique_field):
        """Checks the uniqueness if a unique field is defined

        :param items: list of items to check for uniqueness
        :param unique_field: name of the unique field
        """
        unique_values = []
        for item in items:
            # compare only the active items
            if not item.get("is_active"):
                continue

            if not item.get(unique_field):
                raise SuperdeskApiError.badRequestError("{} cannot be empty".format(unique_field))

            unique_value = str(item.get(unique_field)).upper()

            if unique_value in unique_values:
                raise SuperdeskApiError.badRequestError(
                    "Value {} for field {} is not unique".format(item.get(unique_field), unique_field)
                )

            unique_values.append(unique_value)

    def _filter_inactive_vocabularies(self, item):
        vocs = item["items"]
        active_vocs = (
            {k: voc[k] for k in voc.keys() if k != "is_active"} for voc in vocs if voc.get("is_active", True)
        )

        item["items"] = list(active_vocs)

    def _cast_items(self, vocab):
        """Cast values in vocabulary items using predefined schema.

        :param vocab
        """
        schema = vocab_schema.get(vocab.get("_id"), {})
        for item in vocab.get("items", []):
            for field, field_schema in schema.items():
                if field in item:
                    item[field] = serialize_value(field_schema["type"], item[field])

    def _send_notification(self, updated_vocabulary, event="vocabularies:updated"):
        """
        Sends notification about the updated vocabulary to all the connected clients.
        """

        user = get_user_from_request()
        push_notification(
            event,
            vocabulary=updated_vocabulary.get("display_name"),
            user=str(user[config.ID_FIELD]) if user else None,
            vocabulary_id=updated_vocabulary["_id"],
        )

    def get_rightsinfo(self, item):
        rights_key = item.get("source", item.get("original_source", "default"))
        all_rights = self.find_one(req=None, _id="rightsinfo")
        if not all_rights or not all_rights.get("items"):
            return {}
        try:
            all_rights["items"] = self.get_locale_vocabulary(all_rights.get("items"), item.get("language"))
            default_rights = next(info for info in all_rights["items"] if info["name"] == "default")
        except StopIteration:
            default_rights = None
        try:
            rights = next(info for info in all_rights["items"] if info["name"] == rights_key)
        except StopIteration:
            rights = default_rights
        if rights:
            return {
                "copyrightholder": rights.get("copyrightHolder"),
                "copyrightnotice": rights.get("copyrightNotice"),
                "usageterms": rights.get("usageTerms"),
            }
        else:
            return {}

    def get_extra_fields(self):
        return list(self.get(req=None, lookup={"field_type": {"$exists": True, "$ne": None}}))

    def get_custom_vocabularies(self):
        return list(
            self.get(
                req=None,
                lookup={
                    "field_type": None,
                    "service": {"$exists": True},
                },
            )
        )

    def get_forbiden_custom_vocabularies(self):
        return list(
            self.get(
                req=None,
                lookup={
                    "field_type": None,
                    "selection_type": "do not show",
                    "service": {"$exists": True},
                },
            )
        )

    def get_locale_vocabulary(self, vocabulary, language):
        if not vocabulary or not language:
            return vocabulary
        locale_vocabulary = []
        for item in vocabulary:
            if "translations" not in item:
                locale_vocabulary.append(item)
                continue
            new_item = item.copy()
            locale_vocabulary.append(new_item)
            for field, values in new_item.get("translations", {}).items():
                if field in new_item and language in values:
                    new_item[field] = values[language]
        return locale_vocabulary

    def add_missing_keywords(self, keywords, language=None):
        if not keywords:
            return
        cv = self.find_one(req=None, _id=KEYWORDS_CV)
        if cv:
            existing = {item["name"].lower() for item in cv.get("items", [])}
            missing = [keyword for keyword in keywords if keyword.lower() not in existing]
            if missing:
                updates = {"items": cv.get("items", [])}
                for keyword in missing:
                    updates["items"].append(
                        {
                            "name": keyword,
                            "qcode": keyword,
                            "is_active": True,
                        }
                    )
                self.on_update(updates, cv)
                self.system_update(cv["_id"], updates, cv)
                self.on_updated(updates, cv)
        else:
            items = [
                {
                    "name": keyword,
                    "qcode": keyword,
                    "is_active": True,
                }
                for keyword in keywords
            ]
            cv = {
                "_id": KEYWORDS_CV,
                "items": items,
                "type": "manageable",
                "display_name": _("Keywords"),
                "unique_field": "name",
                "schema": {
                    "name": {},
                    "qcode": {},
                },
            }
            self.post([cv])

    def get_article_cv_item(self, item, scheme):
        article_item = {k: v for k, v in item.items() if k not in ("is_active",)}
        article_item.update({"scheme": scheme})
        return article_item


def is_related_content(item_name, related_content=None):
    if related_content is None:
        related_content = list(
            get_resource_service("vocabularies").get(req=None, lookup={"field_type": "related_content"})
        )

    if related_content and item_name.split("--")[0] in [content["_id"] for content in related_content]:
        return True

    return False



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from unittest.mock import MagicMock

from apps.highlights import init_app
from superdesk.tests import TestCase


class MarkedForHighlightsServiceTest(TestCase):
    """Base class for MarkedForHighlightsService tests."""

    def setUpForChildren(self):
        super().setUpForChildren()
        init_app(self.app)
        try:
            from apps.highlights.service import MarkedForHighlightsService
        except ImportError:
            self.fail("Could not import class under test " "(MarkedForHighlightsService).")
        else:
            self.instance = MarkedForHighlightsService()


@mock.patch("apps.highlights.service.push_notification")
@mock.patch("apps.highlights.service.get_resource_service")
class CreateMethodTestCase(MarkedForHighlightsServiceTest):
    """Tests for the create() method."""

    def setUp(self):
        db_item_1 = {
            "_id": "tag:item_1",
            "highlights": [],
            "_updated": "01.01.1111",
            "_etag": "111",
        }
        db_item_2 = {
            "_id": "tag:item_2",
            "highlights": [],
            "_updated": "02.02.2222",
            "_etag": "222",
        }

        # some of the existing archive items in database
        self.db_items = {"tag:item_1": db_item_1, "tag:item_2": db_item_2}

    def test_pushes_notifications_for_newly_highlighted_items(self, fake_get_service, fake_push_notify):
        def fake_find_one(**kwargs):
            item_id = kwargs.get("_id")
            return self.db_items.get(item_id)

        fake_archive_service = MagicMock()
        fake_archive_service.find_one = fake_find_one

        fake_get_service.return_value = fake_archive_service

        # items' data in HTTP request
        req_item_1 = {"marked_item": "tag:item_1", "highlights": "highlight_X"}
        req_item_2 = {"marked_item": "tag:item_2", "highlights": "highlight_Y"}

        self.instance.create([req_item_1, req_item_2])

        # notifications should have been pushed, one for each highlighted item
        self.assertEqual(fake_push_notify.call_count, 2)
        fake_push_notify.assert_any_call("item:highlights", marked=1, item_id="tag:item_1", mark_id="highlight_X")
        fake_push_notify.assert_any_call("item:highlights", marked=1, item_id="tag:item_2", mark_id="highlight_Y")

    def test_pushes_notifications_for_newly_unhighlighted_items(self, fake_get_service, fake_push_notify):
        # existing items ARE highlighted, and we will test toggling this off
        self.db_items["tag:item_1"]["highlights"] = ["highlight_X"]
        self.db_items["tag:item_2"]["highlights"] = ["highlight_Y"]

        def fake_find_one(**kwargs):
            item_id = kwargs.get("_id")
            return self.db_items.get(item_id)

        fake_archive_service = MagicMock()
        fake_archive_service.find_one = fake_find_one

        fake_get_service.return_value = fake_archive_service

        # items' data in HTTP request
        req_item_1 = {"marked_item": "tag:item_1", "highlights": "highlight_X"}
        req_item_2 = {"marked_item": "tag:item_2", "highlights": "highlight_Y"}

        self.instance.create([req_item_1, req_item_2])

        # notifications should have been pushed, one for each highlighted item
        self.assertEqual(fake_push_notify.call_count, 2)
        fake_push_notify.assert_any_call("item:highlights", marked=0, item_id="tag:item_1", mark_id="highlight_X")
        fake_push_notify.assert_any_call("item:highlights", marked=0, item_id="tag:item_2", mark_id="highlight_Y")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from operator import itemgetter

import superdesk
from superdesk.utils import ListCursor
from superdesk.privilege import get_privilege_list


class PrivilegeService(superdesk.Service):
    def get(self, req, lookup):
        """
        Returns all registered privileges.
        """

        _privileges = get_privilege_list()
        _privileges = sorted(_privileges, key=itemgetter("label"))

        return ListCursor(_privileges)


class PrivilegeResource(superdesk.Resource):
    """Read-only resource with all privileges."""

    resource_methods = ["GET"]
    item_methods = []
    schema = {
        "name": {"type": "string"},
        "label": {"type": "string"},
        "description": {"type": "string"},
    }


def init_app(app):
    PrivilegeResource("privileges", app=app, service=PrivilegeService())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import flask
import logging
from bson import ObjectId
from flask import current_app as app, json, request
from eve.utils import config
from superdesk.activity import add_activity, ACTIVITY_CREATE, ACTIVITY_UPDATE
from superdesk.metadata.item import SIGN_OFF
from superdesk.services import BaseService
from superdesk.utils import is_hashed, get_hash, compare_preferences
from superdesk import get_resource_service
from superdesk.emails import send_user_status_changed_email, send_activate_account_email
from superdesk.utc import utcnow
from superdesk.privilege import get_privilege_list
from superdesk.errors import SuperdeskApiError
from superdesk.users.errors import UserInactiveError, UserNotRegisteredException
from superdesk.notification import push_notification
from superdesk.validation import ValidationError
from superdesk.utils import ignorecase_query

logger = logging.getLogger(__name__)


def get_display_name(user):
    if user.get("first_name") or user.get("last_name"):
        display_name = "%s %s" % (user.get("first_name", ""), user.get("last_name", ""))
        return display_name.strip()
    else:
        return user.get("username")


def is_admin(user):
    """Test if given user is admin.

    :param user
    """
    return user.get("user_type", "user") == "administrator"


def get_admin_privileges():
    """Get privileges for admin user."""
    return dict.fromkeys([p["name"] for p in get_privilege_list()], 1)


def get_privileges(user, role):
    """Get privileges for given user and role.

    :param user
    :param role
    """
    if is_admin(user):
        return get_admin_privileges()

    if role:
        role_privileges = role.get("privileges", {})
        return dict(list(role_privileges.items()) + list(user.get("privileges", {}).items()))

    return user.get("privileges", {})


def current_user_has_privilege(privilege):
    """Test if current user has given privilege.

    In case there is no current user we assume it's system (via worker/manage.py)
    and let it pass.

    :param privilege
    """
    if not getattr(flask.g, "user", None):  # no user - worker can do it
        return True
    privileges = get_privileges(flask.g.user, getattr(flask.g, "role", None))
    return privileges.get(privilege, False)


def is_sensitive_update(updates):
    """Test if given update is sensitive and might change user privileges."""
    return "role" in updates or "privileges" in updates or "user_type" in updates


def get_invisible_stages(user_id):
    user_desks = list(get_resource_service("user_desks").get(req=None, lookup={"user_id": user_id}))
    user_desk_ids = [d["_id"] for d in user_desks]
    return get_resource_service("stages").get_stages_by_visibility(False, user_desk_ids)


def set_sign_off(user):
    """
    Set sign_off property on user if it's not set already.
    """

    if SIGN_OFF not in user or user[SIGN_OFF] is None:
        sign_off_mapping = app.config.get("SIGN_OFF_MAPPING", None)
        if sign_off_mapping and sign_off_mapping in user:
            user[SIGN_OFF] = user[sign_off_mapping]
        elif SIGN_OFF in user and user[SIGN_OFF] is None:
            user[SIGN_OFF] = ""
        elif "first_name" not in user or "last_name" not in user:
            user[SIGN_OFF] = user["username"][:3].upper()
        else:
            user[SIGN_OFF] = "{first_name[0]}{last_name[0]}".format(**user)


def update_sign_off(updates):
    """
    Update sign_off property on user if the mapped field is changed.
    """

    sign_off_mapping = app.config.get("SIGN_OFF_MAPPING", None)
    if sign_off_mapping and sign_off_mapping in updates:
        updates[SIGN_OFF] = updates[sign_off_mapping]


def get_sign_off(user):
    """
    Gets sign_off property on user if it's not set already.
    """

    if SIGN_OFF not in user or user[SIGN_OFF] is None:
        set_sign_off(user)

    return user[SIGN_OFF]


class UsersService(BaseService):

    _updating_stage_visibility = True

    def __is_invalid_operation(self, user, updates, method):
        """Checks if the requested 'PATCH' or 'DELETE' operation is Invalid.

        Operation is invalid if one of the below is True:
            1. Check if the user is updating his/her own status.
            2. Check if the user is changing the role/user_type/privileges of other logged-in users.
            3. A user without 'User Management' privilege is changing status/role/user_type/privileges

        :return: error message if invalid.
        """

        if "user" in flask.g:
            if method == "PATCH":
                if "is_active" in updates or "is_enabled" in updates:
                    if str(user["_id"]) == str(flask.g.user["_id"]):
                        return "Not allowed to change your own status"
                    elif not current_user_has_privilege("users"):
                        return "Insufficient privileges to change user state"
                if (
                    str(user["_id"]) != str(flask.g.user["_id"])
                    and user.get("session_preferences")
                    and is_sensitive_update(updates)
                ):
                    return "Not allowed to change the role/user_type/privileges of a logged-in user"
            elif method == "DELETE" and str(user["_id"]) == str(flask.g.user["_id"]):
                return "Not allowed to disable your own profile."

        if method == "PATCH" and is_sensitive_update(updates) and not current_user_has_privilege("users"):
            return "Insufficient privileges to update role/user_type/privileges"

    def __handle_status_changed(self, updates, user):
        enabled = updates.get("is_enabled", None)
        active = updates.get("is_active", None)

        if enabled is not None or active is not None:
            get_resource_service("auth").delete_action({"username": user.get("username")})  # remove active tokens
            updates["session_preferences"] = {}

            # send email notification
            can_send_mail = get_resource_service("preferences").email_notification_is_enabled(user_id=user["_id"])

            status = ""

            if enabled is not None:
                status = "enabled" if enabled else "disabled"

            if (status == "" or status == "enabled") and active is not None:
                status = "enabled and active" if active else "enabled but inactive"

            if can_send_mail:
                send_user_status_changed_email([user.get("email")], status)

    def __send_notification(self, updates, user):
        user_id = user["_id"]

        if "is_enabled" in updates and not updates["is_enabled"]:
            push_notification("user_disabled", updated=1, user_id=str(user_id))
        elif "is_active" in updates and not updates["is_active"]:
            push_notification("user_inactivated", updated=1, user_id=str(user_id))
        elif "role" in updates:
            push_notification("user_role_changed", updated=1, user_id=str(user_id))
        elif "privileges" in updates:
            added, removed, modified = compare_preferences(user.get("privileges", {}), updates["privileges"])
            if len(removed) > 0 or (1, 0) in modified.values():
                push_notification("user_privileges_revoked", updated=1, user_id=str(user_id))
            if len(added) > 0:
                add_activity(
                    ACTIVITY_UPDATE,
                    "user {{user}} has been granted new privileges: Please re-login.",
                    self.datasource,
                    notify=[user_id],
                    user=user.get("display_name", user.get("username")),
                )
        elif "user_type" in updates:
            if not is_admin(updates):
                push_notification("user_type_changed", updated=1, user_id=str(user_id))
            else:
                add_activity(
                    ACTIVITY_UPDATE,
                    "user {{user}} is updated to administrator: Please re-login.",
                    self.datasource,
                    notify=[user_id],
                    user=user.get("display_name", user.get("username")),
                )
        else:
            push_notification("user", updated=1, user_id=str(user_id))

    def get_avatar_renditions(self, doc):
        renditions = get_resource_service("upload").find_one(req=None, _id=doc)
        return renditions.get("renditions") if renditions is not None else None

    def on_create(self, docs):
        for user_doc in docs:
            user_doc.setdefault("password_changed_on", utcnow())
            user_doc.setdefault("display_name", get_display_name(user_doc))
            user_doc.setdefault(SIGN_OFF, set_sign_off(user_doc))
            user_doc.setdefault("role", get_resource_service("roles").get_default_role_id())
            if user_doc.get("avatar"):
                user_doc.setdefault("avatar_renditions", self.get_avatar_renditions(user_doc["avatar"]))

            get_resource_service("preferences").set_user_initial_prefs(user_doc)

    def on_created(self, docs):
        for user_doc in docs:
            self.__update_user_defaults(user_doc)
            add_activity(
                ACTIVITY_CREATE,
                "created user {{user}}",
                self.datasource,
                user=user_doc.get("display_name", user_doc.get("username")),
            )
            self.update_stage_visibility_for_user(user_doc)

    def on_update(self, updates, original):
        """Overriding the method to:

        1. Prevent user from the below:
            a. Check if the user is updating his/her own status.
            b. Check if the user is changing the status of other logged-in users.
            c. A user without 'User Management' privilege is changing role/user_type/privileges
        2. Set Sign Off property if it's not been set already
        """
        error_message = self.__is_invalid_operation(original, updates, "PATCH")
        if error_message:
            raise SuperdeskApiError.forbiddenError(message=error_message)

        if updates.get("is_enabled", False):
            updates["is_active"] = True

        update_sign_off(updates)

        if updates.get("avatar"):
            updates["avatar_renditions"] = self.get_avatar_renditions(updates["avatar"])

    def on_updated(self, updates, user):
        if "role" in updates or "privileges" in updates:
            get_resource_service("preferences").on_update(updates, user)
        self.__handle_status_changed(updates, user)
        self.__send_notification(updates, user)

    def on_delete(self, user):
        """Overriding the method to prevent user from the below:

        1. Check if the user is updating his/her own status.
        2. Check if the user is changing the status of other logged-in users.
        3. A user without 'User Management' privilege is changing role/user_type/privileges
        """
        updates = {"is_enabled": False, "is_active": False}
        error_message = self.__is_invalid_operation(user, updates, "DELETE")
        if error_message:
            raise SuperdeskApiError.forbiddenError(message=error_message)

    def delete(self, lookup):
        """
        Overriding the method to prevent from hard delete
        """
        user = super().find_one(req=None, _id=str(lookup["_id"]))
        return super().update(
            id=ObjectId(lookup["_id"]), updates={"is_enabled": False, "is_active": False}, original=user
        )

    def __clear_locked_items(self, user_id):
        archive_service = get_resource_service("archive")
        archive_autosave_service = get_resource_service("archive_autosave")

        doc_to_unlock = {
            "lock_user": None,
            "lock_session": None,
            "lock_time": None,
            "lock_action": None,
            "force_unlock": True,
        }
        user = ObjectId(user_id) if isinstance(user_id, str) else user_id
        query = {"$or": [{"lock_user": user}, {"task.user": user, "task.desk": {"$exists": False}}]}

        items_locked_by_user = archive_service.get_from_mongo(req=None, lookup=query)

        if items_locked_by_user and items_locked_by_user.count():
            for item in items_locked_by_user:
                # delete the item if nothing is saved so far
                if item[config.VERSION] == 0 and item["state"] == "draft":
                    get_resource_service("archive").delete(lookup={"_id": item["_id"]})
                else:
                    archive_service.update(item["_id"], doc_to_unlock, item)
                    archive_autosave_service.delete(lookup={"_id": item["_id"]})

    def on_deleted(self, doc):
        """Overriding to add to activity stream and handle user clean up.

        1. Authenticated Sessions
        2. Locked Articles
        3. Reset Password Tokens
        """

        add_activity(
            ACTIVITY_UPDATE,
            "disabled user {{user}}",
            self.datasource,
            user=doc.get("display_name", doc.get("username")),
        )
        self.__clear_locked_items(str(doc["_id"]))
        self.__handle_status_changed(updates={"is_enabled": False, "is_active": False}, user=doc)

    def on_fetched(self, document):
        for doc in document["_items"]:
            self.__update_user_defaults(doc)

    def on_fetched_item(self, doc):
        self.__update_user_defaults(doc)

    def __update_user_defaults(self, doc):
        """Set default fields for users"""
        doc.pop("password", None)

        doc.setdefault("display_name", get_display_name(doc))
        doc.setdefault("is_enabled", doc.get("is_active"))
        doc.setdefault(SIGN_OFF, set_sign_off(doc))
        doc["dateline_source"] = app.config["ORGANIZATION_NAME_ABBREVIATION"]

    def user_is_waiting_activation(self, doc):
        return doc.get("needs_activation", False)

    def is_user_active(self, doc):
        return doc.get("is_active", False)

    def get_role(self, user):
        if user:
            role_id = user.get("role", None)
            if role_id:
                return get_resource_service("roles").find_one(_id=role_id, req=None)
        return None

    def set_privileges(self, user, role):
        user["active_privileges"] = get_privileges(user, role)

    def get(self, req, lookup):
        try:
            is_author = req.args["is_author"]
        except (AttributeError, TypeError, KeyError):
            pass
        else:
            if is_author in ("0", "1"):
                lookup["is_author"] = bool(int(is_author))
            else:
                logger.warn("bad value of is_author argument ({value})".format(value=is_author))

        """filtering out inactive users and disabled users"""

        args = req.args if req and req.args else {}

        # Filtering inactive users
        if not args.get("show_inactive"):
            if lookup is not None:
                lookup["is_active"] = True
            else:
                lookup = {"is_active": True}

        # Filtering disabled users
        if not args.get("show_disabled"):
            if lookup is not None:
                lookup["is_enabled"] = True
            else:
                lookup = {"is_enabled": True}

        return super().get(req, lookup)

    def get_users_by_user_type(self, user_type="user"):
        return list(self.get(req=None, lookup={"user_type": user_type}))

    def get_users_by_role(self, role_id):
        return list(self.get(req=None, lookup={"role": role_id}))

    def get_invisible_stages(self, user_id):
        return get_invisible_stages(user_id) if user_id else []

    def get_invisible_stages_ids(self, user_id):
        return [str(stage["_id"]) for stage in self.get_invisible_stages(user_id)]

    def get_user_by_email(self, email_address):
        """Finds a user by the given email_address.

        Does a exact match.

        :param email_address:
        :type email_address: str with valid email format
        :return: user object if found.
        :rtype: dict having user details :py:class: `superdesk.users.users.UsersResource`
        :raises: UserNotRegisteredException if no user found with the given email address.
        """
        user = self.find_one(req=None, email=email_address)
        if not user:
            raise UserNotRegisteredException("No user registered with email %s" % email_address)

        return user

    def update_stage_visibility_for_users(self):
        if not self._updating_stage_visibility:
            return
        logger.info("Updating Stage Visibility Started")
        users = list(get_resource_service("users").get(req=None, lookup=None))
        for user in users:
            self.update_stage_visibility_for_user(user)

        logger.info("Updating Stage Visibility Completed")

    def update_stage_visibility_for_user(self, user):
        if not self._updating_stage_visibility:
            return
        try:
            logger.info("Updating Stage Visibility for user {}.".format(user.get(config.ID_FIELD)))
            stages = self.get_invisible_stages_ids(user.get(config.ID_FIELD))
            self.system_update(user.get(config.ID_FIELD), {"invisible_stages": stages}, user)
            user["invisible_stages"] = stages
            logger.info("Updated Stage Visibility for user {}.".format(user.get(config.ID_FIELD)))
        except Exception:
            logger.exception("Failed to update the stage visibility " "for user: {}".format(user.get(config.ID_FIELD)))

    def stop_updating_stage_visibility(self):
        if not app.config.get("SUPERDESK_TESTING"):
            raise RuntimeError("Only allowed during testing")
        self._updating_stage_visibility = False

    def start_updating_stage_visibility(self):
        self._updating_stage_visibility = True


class DBUsersService(UsersService):
    """
    Service class for UsersResource and should be used when AD is inactive.
    """

    def on_create(self, docs):
        super().on_create(docs)
        for doc in docs:
            if doc.get("password", None) and not is_hashed(doc.get("password")):
                doc["password"] = get_hash(doc.get("password"), app.config.get("BCRYPT_GENSALT_WORK_FACTOR", 12))

    def on_created(self, docs):
        """Send email to user with reset password token."""
        super().on_created(docs)
        resetService = get_resource_service("reset_user_password")
        activate_ttl = app.config["ACTIVATE_ACCOUNT_TOKEN_TIME_TO_LIVE"]
        for doc in docs:
            if self.user_is_waiting_activation(doc):
                tokenDoc = {"user": doc["_id"], "email": doc["email"]}
                id = resetService.store_reset_password_token(tokenDoc, doc["email"], activate_ttl, doc["_id"])
                if not id:
                    raise SuperdeskApiError.internalError("Failed to send account activation email.")
                tokenDoc.update({"username": doc["username"]})
                send_activate_account_email(tokenDoc, activate_ttl)

    def on_update(self, updates, user):
        super().on_update(updates, user)
        if updates.get("first_name") or updates.get("last_name"):
            updated_user = {
                "first_name": user.get("first_name", ""),
                "last_name": user.get("last_name", ""),
                "username": user.get("username", ""),
            }
            if updates.get("first_name"):
                updated_user["first_name"] = updates.get("first_name")
            if updates.get("last_name"):
                updated_user["last_name"] = updates.get("last_name")
            updates["display_name"] = get_display_name(updated_user)

    def update_password(self, user_id, password):
        """Update the user password.

        Returns true if successful.
        """
        user = self.find_one(req=None, _id=user_id)

        if not user:
            raise SuperdeskApiError.unauthorizedError("User not found")

        if not self.is_user_active(user):
            raise UserInactiveError()

        updates = {
            "password": get_hash(password, app.config.get("BCRYPT_GENSALT_WORK_FACTOR", 12)),
            "password_changed_on": utcnow(),
            app.config["LAST_UPDATED"]: utcnow(),
        }

        if self.user_is_waiting_activation(user):
            updates["needs_activation"] = False

        self.patch(user_id, updates=updates)

    def on_deleted(self, doc):
        """
        Overriding clean up reset password tokens:
        """

        super().on_deleted(doc)
        get_resource_service("reset_user_password").remove_all_tokens_for_email(doc.get("email"))

    def _process_external_data(self, _data, update=False):
        data = _data.copy()
        if data.get("role"):
            role_name = data.pop("role")
            role = get_resource_service("roles").find_one(req=None, name=ignorecase_query(role_name))
            if role:
                data["role"] = role["_id"]
        if not update and (data.get("desk") or app.config.get("USER_EXTERNAL_DESK")):
            desk_name = data.pop("desk", None) or app.config.get("USER_EXTERNAL_DESK")
            desk = get_resource_service("desks").find_one(req=None, name=ignorecase_query(desk_name))
            if desk:
                data["desk"] = desk["_id"]
        data["needs_activation"] = False
        if update:
            data.pop("desk", None)
            data.pop("email", None)
            data.pop("username", None)
        elif data.get("username"):
            if app.config.get("USER_EXTERNAL_USERNAME_STRIP_DOMAIN"):
                data["username"] = data["username"].split("@")[0]
            data["username"] = data["username"].replace("@", ".")  # @ breaks mentioning
        validator = self._validator()
        if not validator.validate(data, update=update):
            raise ValidationError(validator.errors)
        return validator.normalized(data) if not update else data

    def create_external_user(self, data):
        docs = [self._process_external_data(data)]
        self.on_create(docs)
        self.create(docs)
        for user in docs:
            if user.get("desk"):
                get_resource_service("desks").add_member(user["desk"], user["_id"])
        return docs[0]

    def update_external_user(self, _id, data):
        orig = self.find_one(req=None, _id=ObjectId(_id))
        updates = self._process_external_data(data, update=True)
        self.system_update(ObjectId(_id), updates, orig)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime
import dateutil.parser
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from lxml import etree
from superdesk.errors import ParserError
from superdesk.utc import utc
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, FORMAT, FORMATS
from superdesk.text_utils import get_word_count
from superdesk.io.iptc import subject_codes
from superdesk.errors import SkipValue
import re
import superdesk

SETTINGS_MAPPING_PARAM = "NITF_MAPPING"
SUBJECT_TYPE = "tobject.subject.type"
SUBJECT_MATTER = "tobject.subject.matter"
SUBJECT_DETAIL = "tobject.subject.detail"

subject_fields = (SUBJECT_TYPE, SUBJECT_MATTER, SUBJECT_DETAIL)


class NITFFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse if the feed is in NITF format.
    """

    # FIXME: some behaviour here are specific to AAP (e.g. getPlace)
    #        they have been kept to avoid breaking AAP, but they can
    #        now be moved to Superdesk's settings in aap branch

    NAME = "nitf"

    label = "NITF Parser"

    def __init__(self):
        super().__init__()
        self.default_mapping = {
            "guid": {"xpath": "head/docdata/doc-id/@id-string", "default": None},
            "uri": {"xpath": "head/docdata/doc-id/@id-string", "default": None},
            "urgency": {
                "xpath": "head/docdata/urgency/@ed-urg",
                "default_attr": 5,
                "filter": int,
            },
            "pubstatus": {
                "xpath": "head/docdata/@management-status",
                "default_attr": "usable",
            },
            "firstcreated": {
                "xpath": "head/docdata/date.issue",
                "filter": self.get_norm_datetime,
            },
            "versioncreated": {
                "xpath": "head/docdata/date.issue",
                "filter": self.get_norm_datetime,
            },
            "expiry": {
                "xpath": "head/docdata/date.expire",
                "filter": self.get_norm_datetime,
            },
            "subject": self.get_subjects,
            "body_html": self.get_content,
            FORMAT: self.get_format,
            "place": self.get_place,
            "keywords": {
                "xpath": "head/docdata",
                "filter": self.get_keywords,
            },
            "genre": self.get_genre,
            "ednote": "head/docdata/ed-msg/@info",
            "headline": self.get_headline,
            "abstract": self.get_abstract,
            "byline": self.get_byline,
            # metadata
            "slugline": "head/meta[@name='anpa-keyword']/@content",
            "ingest_provider_sequence": "head/meta[@name='anpa-sequence']/@content",
            "anpa_category": {
                "xpath": "head/meta[@name='anpa-category']",
                "filter": lambda elem: [{"qcode": elem.get("content"), "name": ""}],
            },
            "word_count": {
                "xpath": "head/meta[@name='anpa-wordcount']",
                "filter": lambda elem: int(elem.get("content")),
            },
            "anpa_take_key": "head/meta[@name='anpa-keyword']",
            "priority": {
                "xpath": "head/meta[@name='aap-priority']",
                "filter": lambda elem: self.map_priority(elem.get("content")),
            },
            "original_creator": self.get_original_creator,
            "version_creator": self.get_version_creator,
            "original_source": "head/meta[@name='aap-source']/@content",
            "source": "head/meta[@name='aap-source']/@content",
            "task": self.get_task,
        }

    def can_parse(self, xml):
        return xml.tag == "nitf"

    def parse(self, xml, provider=None):
        item = {
            ITEM_TYPE: CONTENT_TYPE.TEXT,  # set the default type.
        }
        try:
            self.do_mapping(item, xml, SETTINGS_MAPPING_PARAM)
            elem = xml.find("body/body.head/dateline/location/city")
            if elem is not None:
                self.set_dateline(item, city=elem.text)

            item.setdefault("word_count", get_word_count(item["body_html"], no_html=True))
        except Exception as ex:
            raise ParserError.nitfParserError(ex, provider)
        return item

    def get_norm_datetime(self, tree):
        if tree is None:
            return

        try:
            value = datetime.strptime(tree.attrib["norm"], "%Y%m%dT%H%M%S")
        except ValueError:
            try:
                value = datetime.strptime(tree.attrib["norm"], "%Y%m%dT%H%M%S%z")
            except ValueError:
                try:
                    value = dateutil.parser.parse(tree.attrib["norm"])
                except ValueError:
                    return

        return utc.normalize(value) if value.tzinfo else value

    def get_subjects(self, tree):
        """Finds all the subject tags in the passed tree and returns the parsed subjects.

        All entries will have both the name and qcode populated.

        :param tree:
        :return: a list of subject dictionaries
        """
        subjects = []
        qcodes = []  # we check qcodes to avoid duplicates
        for elem in tree.findall("head/tobject/tobject.subject"):
            qcode = elem.get("tobject.subject.refnum")
            if qcode in qcodes:
                # we ignore duplicates
                continue
            else:
                qcodes.append(qcode)
            for field in subject_fields:
                if elem.get(field):
                    if field == SUBJECT_TYPE:
                        field_qcode = qcode[:2] + "000000"
                    elif field == SUBJECT_MATTER:
                        field_qcode = qcode[:5] + "000"
                    else:
                        field_qcode = qcode

                    if subject_codes.get(field_qcode) and not any(c["qcode"] == field_qcode for c in subjects):
                        subjects.append({"name": elem.get(field), "qcode": field_qcode})

            # if the subject_fields are not specified.
            if not any(c["qcode"] == qcode for c in subjects) and subject_codes.get(qcode):
                subjects.append({"name": subject_codes[qcode], "qcode": qcode})
        return subjects

    def get_anpa_format(self, xml):
        elem = xml.find("head/meta[@name='anpa-format']")
        if elem is not None:
            content = elem.get("content")
            return content.lower() if content is not None else "x"

    def parse_to_preformatted(self, element):
        """Extract the content of the element as a plain string with line enders

        :param element:
        :return:
        """
        elements = []
        parsed = etree.fromstring("<div>" + element + "</div>")
        for elem in parsed.iterfind(".//"):
            text = etree.tostring(elem, encoding="unicode", method="text")
            elements.append(text + "\r\n")
        return "".join(elements)

    def get_content(self, xml):
        elements = []
        for elem in xml.find("body/body.content"):
            elements.append(etree.tostring(elem, encoding="unicode"))
        content = "".join(elements)
        if self.get_anpa_format(xml) == "t":
            if not content.startswith("<pre>"):
                # convert content to text in a pre tag
                content = "<pre>{}</pre>".format(self.parse_to_preformatted(content))
            else:
                content = self.parse_to_preformatted(content)
        return content

    def get_format(self, xml):
        anpa_format = self.get_anpa_format(xml)
        if anpa_format is not None:
            return FORMATS.PRESERVED if anpa_format == "t" else FORMATS.HTML

        body_elem = xml.find("body/body.content")
        # if the body contains only a single pre tag we mark the format as preserved
        if len(body_elem) == 1 and body_elem[0].tag == "pre":
            return FORMATS.PRESERVED
        else:
            return FORMATS.HTML

    def get_place(self, tree):
        elem = tree.find("head/meta[@name='aap-place']")
        if elem is None:
            return self.get_places(tree.find("head/docdata"))
        locator_map = superdesk.get_resource_service("vocabularies").find_one(req=None, _id="locators")
        return [x for x in locator_map.get("items", []) if x["qcode"] == elem.get("content")]

    def get_places(self, docdata):
        places = []
        evloc = docdata.find("evloc")
        if evloc is not None and evloc.attrib.get("city") and evloc.attrib.get("iso-cc"):
            places.append(
                {
                    "name": evloc.attrib.get("city"),
                    "code": evloc.attrib.get("iso-cc"),
                    "qcode": evloc.attrib.get("city"),
                }
            )
        return places

    def get_keywords(self, docdata):
        return [keyword.attrib["key"] for keyword in docdata.findall("key-list/keyword") if "key" in keyword.attrib]

    def get_genre(self, tree):
        elem = tree.find("head/tobject/tobject.property")
        if elem is None:
            raise SkipValue()
        genre = elem.get("tobject.property.type")
        genre_map = superdesk.get_resource_service("vocabularies").find_one(req=None, _id="genre")
        if genre_map is not None:
            return [x for x in genre_map.get("items", []) if x["name"] == genre]
        else:
            raise SkipValue()

    def get_headline(self, xml):
        if xml.find("body/body.head/hedline/hl1") is not None:
            return xml.find("body/body.head/hedline/hl1").text
        else:
            if xml.find("head/title") is not None:
                return xml.find("head/title").text
        raise SkipValue()

    def get_abstract(self, xml):
        elem = xml.find("body/body.head/abstract/p")
        if elem is None:
            elem = xml.find("body/body.head/abstract")
        return elem.text if elem is not None else ""

    def get_byline(self, tree):
        elem = tree.find("body/body.head/byline")
        byline = ""
        if elem is not None:
            byline = elem.text
            person = elem.find("person")
            if person is not None and person.text is not None:
                byline = "{} {}".format(byline.strip(), person.text.strip())
        return byline

    def get_original_creator(self, tree):
        elem = tree.find("head/meta[@name='aap-original-creator']")
        if elem is not None:
            query = {"username": re.compile("^{}$".format(elem.get("content")), re.IGNORECASE)}
            user = superdesk.get_resource_service("users").find_one(req=None, **query)
            if user is not None:
                return user.get("_id")
        raise SkipValue()

    def get_version_creator(self, tree):
        elem = tree.find("head/meta[@name='aap-version-creator']")
        if elem is not None:
            query = {"username": re.compile("^{}$".format(elem.get("content")), re.IGNORECASE)}
            user = superdesk.get_resource_service("users").find_one(req=None, **query)
            if user:
                return user.get("_id")
        raise SkipValue()

    def get_task(self, tree):
        desk_name = tree.find('head/meta[@name="aap-desk"]')
        if desk_name is not None:
            desk = superdesk.get_resource_service("desks").find_one(req=None, name=desk_name.get("content"))
            if desk:
                task = {"desk": desk.get("_id")}
                stage_name = tree.find('head/meta[@name="aap-stage"]')
                if stage_name is not None:
                    lookup = {"$and": [{"name": stage_name.get("content")}, {"desk": str(desk.get("_id"))}]}
                    stages = superdesk.get_resource_service("stages").get(req=None, lookup=lookup)
                    if stages is not None and stages.count() == 1:
                        task["stage"] = stages[0].get("_id")
                return task
        raise SkipValue()


register_feed_parser(NITFFeedParser.NAME, NITFFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import requests_mock
from flask import json
from behave import when, then  # @UnresolvedImport
from apps.publish.enqueue import enqueue_published
from superdesk.tests.steps import assert_200, apply_placeholders, json_match, get_json_data, test_json, format_items
from wooper.general import fail_and_print_body
from wooper.assertions import assert_equal
from superdesk.publish import transmit


@when("we enqueue published")
def step_impl_when_auth(context):
    enqueue_published.apply_async()


@then("we get formatted item")
def then_we_get_formatted_item(context):
    assert_200(context.response)
    try:
        response_data = json.loads(context.response.get_data())
        formatted_item = json.loads(response_data.get("formatted_item", ""))
    except Exception:
        fail_and_print_body(context.response, "response does not contain a valid formatted_item field")
    context_data = json.loads(apply_placeholders(context, context.text))
    assert_equal(
        json_match(context_data, formatted_item), True, msg=str(context_data) + "\n != \n" + str(formatted_item)
    )


@then("we get {total_count} queued items")
def then_we_get_formatted_items(context, total_count):
    assert_200(context.response)
    data = get_json_data(context.response)
    int_count = int(total_count.replace("+", "").replace("<", ""))

    if "+" in total_count:
        assert int_count <= data["_meta"]["total"], "%d items is not enough" % data["_meta"]["total"]
    elif total_count.startswith("<"):
        assert int_count > data["_meta"]["total"], "%d items is too much" % data["_meta"]["total"]
    else:
        assert int_count == data["_meta"]["total"], "got %d: %s" % (
            data["_meta"]["total"],
            format_items(data["_items"]),
        )
    if context.text:
        test_json(context, ["formatted_item"])


@then("we pushed 1 item")
def then_we_pushed_1_item(context):
    return then_we_pushed_x_items(context, 1)


@then("we pushed {count} items")
def then_we_pushed_x_items(context, count):
    history = context.http_mock.request_history
    assert count == len(history), "there were %d calls" % (len(history),)
    if context.text:
        context_data = json.loads(apply_placeholders(context, context.text))
        for i, _ in enumerate(context_data):
            assert_equal(json_match(context_data[i], history[i].json()), True, msg="item[%d]: %s" % (i, history[i]))


@when("we transmit published")
def when_we_transmit_published(context):
    with requests_mock.Mocker() as m:
        context.http_mock = m
        m.post("mock://publish", text=json.dumps({}))
        m.post("mock://assets", text=json.dumps({}))
        transmit.apply_async()
        transmit.apply_async()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.tests import TestCase
from superdesk.publish import init_app, SUBSCRIBER_TYPES
from superdesk.publish.transmitters.odbc import ODBCPublishService


class ODBCTests(TestCase):
    subscribers = [
        {
            "_id": "1",
            "name": "Test",
            "subscriber_type": SUBSCRIBER_TYPES.WIRE,
            "media_type": "media",
            "is_active": True,
            "sequence_num_settings": {"max": 10, "min": 1},
            "critical_errors": {"9004": True},
            "destinations": [
                {
                    "name": "AAP IPNEWS",
                    "delivery_type": "odbc",
                    "format": "AAP IPNEWS",
                    "config": {"stored_procedure": "InsertNews"},
                }
            ],
        }
    ]

    queue_items = [
        {
            "_id": "1",
            "state": "pending",
            "content_type": "text",
            "headline": "test",
            "unique_name": "#2034",
            "publishing_action": "published",
            "published_seq_num": 4,
            "destination": {
                "name": "AAP IPNEWS",
                "delivery_type": "odbc",
                "format": "AAP IPNEWS",
                "config": {"stored_procedure": "InsertNews"},
            },
            "formatted_item": {
                "ident": "0",
                "selector_codes": "3**",
                "wordcount": 313,
                "texttab": "x",
                "originator": "AAP",
                "service_level": "a",
                "keyword": "ROSS",
                "subject": "crime, law and justice",
                "category": "a",
                "take_key": "Take-that",
                "subject_detail": "international court or tribunal",
                "subject_reference": "02011001",
                "article_text": "THIS IS A TEST PLEASE IGNORE",
                "priority": "u",
                "headline": "TEST HEADLINE",
                "usn": 68147,
                "subject_matter": "international law",
                "sequence": 117,
                "news_item_type": "News",
                "author": "",
                "genre": "Current",
                "fullStory": 1,
            },
            "subscriber_id": "1",
            "item_id": "1",
            "item_version": 6,
        }
    ]

    def setUp(self):
        self.subscribers[0]["destinations"][0]["config"]["connection_string"] = superdesk.app.config[
            "ODBC_TEST_CONNECTION_STRING"
        ]
        self.app.data.insert("subscribers", self.subscribers)

        self.queue_items[0]["destination"]["config"]["connection_string"] = superdesk.app.config[
            "ODBC_TEST_CONNECTION_STRING"
        ]
        self.app.data.insert("publish_queue", self.queue_items)
        init_app(self.app)

    def test_transmit(self):
        if superdesk.app.config["ODBC_PUBLISH"]:
            subscriber = self.app.data.find("subscribers", None, None)[0]

            publish_service = ODBCPublishService()
            ret = publish_service._transmit(self.queue_items[0], subscriber)
            self.assertGreater(ret, 0)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import logging
import datetime

from copy import deepcopy
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FeedParser
from superdesk.utc import utc
from superdesk.metadata.utils import generate_tag_from_url

logger = logging.getLogger(__name__)


class NINJSFeedParser(FeedParser):
    """
    Feed Parser for NINJS format
    """

    NAME = "ninjs"

    label = "NINJS Feed Parser"

    direct_copy_properties = (
        "usageterms",
        "language",
        "headline",
        "copyrightnotice",
        "urgency",
        "pubstatus",
        "mimetype",
        "copyrightholder",
        "ednote",
        "body_text",
        "body_html",
        "slugline",
        "keywords",
        "extra",
        "byline",
        "description_text",
        "profile",
    )

    items = []

    def __init__(self):
        super().__init__()

    def can_parse(self, file_path):
        try:
            with open(file_path, "r") as f:
                ninjs = json.load(f)
                if ninjs.get("uri") or ninjs.get("guid"):
                    return True
        except Exception:
            pass
        return False

    def parse(self, file_path, provider=None):
        self.items = []
        with open(file_path, "r") as f:
            ninjs = json.load(f)

        self.items.append(self._transform_from_ninjs(ninjs))
        return self.items

    def _transform_from_ninjs(self, ninjs):
        guid = ninjs.get("guid")
        if not guid and ninjs.get("uri"):
            guid = generate_tag_from_url(ninjs["uri"], "urn")
        item = {"guid": guid, "type": ninjs.get("type"), "uri": ninjs.get("uri")}

        for copy_property in self.direct_copy_properties:
            if ninjs.get(copy_property) is not None:
                item[copy_property] = ninjs[copy_property]

        if ninjs.get("source"):
            item["original_source"] = ninjs["source"]

        if ninjs.get("priority"):
            item["priority"] = int(ninjs["priority"])
        else:
            ninjs["priority"] = 5

        if ninjs.get("genre"):
            item["genre"] = self._format_qcodes(ninjs["genre"])

        if ninjs.get("service"):
            item["anpa_category"] = self._format_qcodes(ninjs["service"])

        if ninjs.get("subject"):
            item["subject"] = self._format_qcodes(ninjs["subject"])

        if ninjs.get("versioncreated"):
            item["versioncreated"] = self.datetime(ninjs.get("versioncreated"))

        if ninjs.get("firstcreated"):
            item["firstcreated"] = self.datetime(ninjs.get("firstcreated"))

        if ninjs.get("associations"):
            item["associations"] = {}

        for key, associated_item in ninjs.get("associations", {}).items():
            if associated_item:
                self.items.append(self._transform_from_ninjs(associated_item))
                if associated_item.get("type") == "picture" and associated_item.get("body_text"):
                    associated_item["alt_text"] = associated_item.get("body_text")
                if associated_item.get("versioncreated"):
                    associated_item["versioncreated"] = self.datetime(associated_item["versioncreated"])
                item["associations"][key] = deepcopy(associated_item)

        if ninjs.get("renditions", {}).get("baseImage"):
            item["renditions"] = {"baseImage": {"href": ninjs.get("renditions", {}).get("original", {}).get("href")}}

        if ninjs.get("located"):
            item["dateline"] = {"located": {"city": ninjs.get("located")}}

        if ninjs.get("type") == "picture" and ninjs.get("body_text"):
            item["alt_text"] = ninjs.get("body_text")

        if ninjs.get("type") == "text" and ninjs.get("description_text"):
            item["abstract"] = ninjs.get("description_text")

        if ninjs.get("place"):
            item["place"] = self._format_qcodes(ninjs["place"])

        if ninjs.get("authors"):
            item["authors"] = self._parse_authors(ninjs["authors"])

        if not item.get("body_html") and ninjs.get("body_xhtml"):
            item["body_html"] = ninjs["body_xhtml"]

        return item

    def _format_qcodes(self, items):
        subjects = []
        for item in items:
            subject = {"name": item.get("name"), "qcode": item.get("code")}
            if item.get("scheme"):
                subject["scheme"] = item.get("scheme")
            subjects.append(subject)

        return subjects

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, "%Y-%m-%dT%H:%M:%S+0000").replace(tzinfo=utc)
        except ValueError:
            return datetime.datetime.strptime(string, "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=utc)

    def _parse_authors(self, authors):
        return [self._parse_author(author) for author in authors]

    def _parse_author(self, author):
        parsed = {
            "name": author["name"],
            "role": author.get("role", ""),
        }

        if author.get("avatar_url"):
            parsed["avatar_url"] = author["avatar_url"]

        if author.get("biography"):
            parsed["biography"] = author["biography"]

        return parsed


register_feed_parser(NINJSFeedParser.NAME, NINJSFeedParser())
